@@ -11,7 +11,7 @@ let setup =
      Path.External.mkdir_p (Path.External.relative tmp deep_path);
      Path.set_root tmp;
      Path.Build.set_build_dir (Path.Build.Kind.of_string "_build");
-     let _conf = Dune_load.load ~ancestor_vcs:None () in
+     let _conf = Dune_load.load ~ancestor_vcs:None in
      let path = Path.Source.of_string deep_path in
      at_exit (fun () -> Sys.remove "./dune-project");
      path)
