@@ -2,8 +2,7 @@ open Stdune
 open Dune
 include Cmdliner.Arg
 
-let package_name =
-  conv ((fun p -> Ok (Package.Name.of_string p)), Package.Name.pp)
+let package_name = conv Package.Name.conv
 
 module Path = struct
   type t = string
@@ -17,10 +16,7 @@ end
 
 let path = Path.conv
 
-let profile =
-  conv
-    ( (fun p -> Ok (Profile.of_string p))
-    , fun fmt t -> Format.pp_print_string fmt (Profile.to_string t) )
+let profile = conv Dune.Profile.conv
 
 module Dep = struct
   module Dep_conf = Dep_conf
@@ -59,7 +55,8 @@ module Dep = struct
       Some (recursive, dir, alias)
 
   let dep_parser =
-    Dune_lang.Syntax.set Stanza.syntax Stanza.latest_version Dep_conf.decode
+    Dune_lang.Syntax.set Stanza.syntax (Active Stanza.latest_version)
+      Dep_conf.decode
 
   let parser s =
     match parse_alias s with
@@ -119,8 +116,11 @@ let bytes =
     | exception User_error.E msg ->
       Result.Error (`Msg (User_message.to_string msg))
   in
-  conv (decode, Format.pp_print_int)
+  let pp_print_int64 state i =
+    Format.pp_print_string state (Int64.to_string i)
+  in
+  conv (decode, pp_print_int64)
 
-let context_name =
-  let printer ppf t = Format.pp_print_string ppf (Context_name.to_string t) in
-  (Context_name.arg_parse, printer)
+let context_name : Context_name.t conv = conv Context_name.conv
+
+let lib_name = conv Dune.Lib_name.conv
