@@ -15,15 +15,23 @@ end
 
 open Let_syntax
 
+module Only_packages = struct
+  type t =
+    { names : Dune.Package.Name.Set.t
+    ; command_line_option : string
+    }
+end
+
 type t =
   { debug_dep_path : bool
   ; debug_findlib : bool
   ; debug_backtraces : bool
+  ; debug_artifact_substitution : bool
   ; profile : Profile.t option
   ; workspace_file : Arg.Path.t option
   ; root : Workspace_root.t
   ; target_prefix : string
-  ; only_packages : Dune.Package.Name.Set.t option
+  ; only_packages : Only_packages.t option
   ; capture_outputs : bool
   ; x : Dune.Context_name.t option
   ; diff_command : string option
@@ -41,6 +49,7 @@ type t =
   ; stats_trace_file : string option
   ; always_show_command_line : bool
   ; promote_install_files : bool
+  ; instrument_with : Dune.Lib_name.t list option
   }
 
 let workspace_file t = t.workspace_file
@@ -63,17 +72,20 @@ let default_target t = t.default_target
 
 let prefix_target common s = common.target_prefix ^ s
 
+let instrument_with t = t.instrument_with
+
 let set_dirs c =
   if c.root.dir <> Filename.current_dir_name then Sys.chdir c.root.dir;
   Path.set_root (Path.External.cwd ());
   Path.Build.set_build_dir (Path.Build.Kind.of_string c.build_dir)
 
 let set_common_other ?log_file c ~targets =
-  Console.init c.config.display;
-  Log.init () ?file:log_file;
+  Config.init c.config;
+  Dune_util.Log.init () ?file:log_file;
   Clflags.debug_dep_path := c.debug_dep_path;
   Clflags.debug_findlib := c.debug_findlib;
-  Clflags.debug_backtraces := c.debug_backtraces;
+  Clflags.debug_backtraces c.debug_backtraces;
+  Clflags.debug_artifact_substitution := c.debug_artifact_substitution;
   Clflags.capture_outputs := c.capture_outputs;
   Clflags.diff_command := c.diff_command;
   Clflags.promote := c.promote;
@@ -92,7 +104,9 @@ let set_common_other ?log_file c ~targets =
   Clflags.ignore_promoted_rules := c.ignore_promoted_rules;
   Option.iter ~f:Dune.Stats.enable c.stats_trace_file
 
-let set_common ?log_file c ~targets =
+let set_common ?log_file ?external_lib_deps_mode c ~targets =
+  Option.iter external_lib_deps_mode ~f:(fun x ->
+      Clflags.external_lib_deps_mode := x);
   set_dirs c;
   set_common_other ?log_file c ~targets
 
@@ -104,6 +118,23 @@ let footer =
 
 let copts_sect = "COMMON OPTIONS"
 
+let examples = function
+  | [] -> `Blocks []
+  | _ :: _ as examples ->
+    let block_of_example index (intro, ex) =
+      let prose = `I (Int.to_string (index + 1) ^ ".", String.trim intro ^ ":")
+      and code_lines =
+        ex |> String.trim |> String.split_lines
+        |> List.concat_map ~f:(fun codeline ->
+               [ `Noblank; `Pre ("      " ^ codeline) ])
+        (* suppress initial blank *)
+        |> List.tl
+      in
+      `Blocks (prose :: code_lines)
+    in
+    let example_blocks = examples |> List.mapi ~f:block_of_example in
+    `Blocks (`S Cmdliner.Manpage.s_examples :: example_blocks)
+
 let help_secs =
   [ `S copts_sect
   ; `P "These options are common to all commands."
@@ -151,7 +182,7 @@ let build_info =
       |> List.map ~f:(fun lib ->
              ( B.Statically_linked_library.name lib
              , ver_string (B.Statically_linked_library.version lib) ))
-      |> List.sort ~compare
+      |> List.sort ~compare:Poly.compare
     in
     ( match libs with
     | [] -> ()
@@ -165,7 +196,7 @@ let build_info =
 module Options_implied_by_dash_p = struct
   type t =
     { root : string option
-    ; only_packages : string option
+    ; only_packages : Only_packages.t option
     ; ignore_promoted_rules : bool
     ; config_file : config_file
     ; profile : Profile.t option
@@ -197,6 +228,24 @@ module Options_implied_by_dash_p = struct
     in
     Option.value x ~default:Default
 
+  let packages =
+    let parser s =
+      let parse_one s =
+        match Package.Name.of_string_opt s with
+        | Some x -> Ok x
+        | None ->
+          ksprintf (fun s -> Error (`Msg s)) "Invalid package name: %S" s
+      in
+      String.split s ~on:',' |> List.map ~f:parse_one |> Result.List.all
+      |> Result.map ~f:Package.Name.Set.of_list
+    in
+    let printer ppf set =
+      Format.pp_print_string ppf
+        (String.concat ~sep:","
+           (Package.Name.Set.to_list set |> List.map ~f:Package.Name.to_string))
+    in
+    Arg.conv ~docv:"PACKAGES" (parser, printer)
+
   let options =
     let+ root =
       Arg.(
@@ -204,23 +253,10 @@ module Options_implied_by_dash_p = struct
         & opt (some dir) None
         & info [ "root" ] ~docs ~docv:"DIR"
             ~doc:
-              {|Use this directory as workspace root instead of
-                      guessing it. Note that this option doesn't change
-                      the interpretation of targets given on the command
-                      line. It is only intended for scripts.|})
-    and+ only_packages =
-      Arg.(
-        value
-        & opt (some string) None
-        & info [ "only-packages" ] ~docs ~docv:"PACKAGES"
-            ~doc:
-              {|Ignore stanzas referring to a package that is not in
-                      $(b,PACKAGES). $(b,PACKAGES) is a comma-separated list
-                      of package names. Note that this has the same effect
-                      as deleting the relevant stanzas from dune files.
-                      It is mostly meant for releases. During development,
-                      it is likely that what you want instead is to
-                      build a particular $(b,<package>.install) target.|})
+              "Use this directory as workspace root instead of guessing it. \
+               Note that this option doesn't change the interpretation of \
+               targets given on the command line. It is only intended for \
+               scripts.")
     and+ ignore_promoted_rules =
       Arg.(
         value & flag
@@ -228,11 +264,9 @@ module Options_implied_by_dash_p = struct
             [ "ignore-promoted-rules" ]
             ~docs
             ~doc:
-              "Ignore rules with (mode promote),\n\
-              \                     except ones with (only ...). The variable\n\
-              \                     %{ignoring_promoted_rules} in dune files \
-               reflects\n\
-              \                     whether this option was passed or not.")
+              "Ignore rules with (mode promote), except ones with (only ...). \
+               The variable %{ignoring_promoted_rules} in dune files reflects \
+               whether this option was passed or not.")
     and+ config_file = config_file_term
     and+ default_target =
       Arg.(
@@ -255,7 +289,7 @@ module Options_implied_by_dash_p = struct
       Arg.(value & flag & info [ "promote-install-files" ] ~docs ~doc)
     in
     { root
-    ; only_packages
+    ; only_packages = None
     ; ignore_promoted_rules
     ; config_file
     ; profile = None
@@ -264,24 +298,9 @@ module Options_implied_by_dash_p = struct
     ; promote_install_files
     }
 
-  let for_release = "for-release-of-packages"
-
-  let dash_p =
-    let+ pkgs =
-      Arg.(
-        value
-        & opt (some string) None
-        & info [ "p"; for_release ] ~docs ~docv:"PACKAGES"
-            ~doc:
-              {|Shorthand for $(b,--root . --only-packages PACKAGE
-                      --ignore-promoted-rules --no-config --profile release).
-                      You must use this option in your $(i,<package>.opam)
-                      files, in order to build only what's necessary when
-                      your project contains multiple packages as well as
-                      getting reproducible builds.|})
-    in
+  let release_options =
     { root = Some "."
-    ; only_packages = pkgs
+    ; only_packages = None
     ; ignore_promoted_rules = true
     ; config_file = No_config
     ; profile = Some Profile.Release
@@ -291,6 +310,66 @@ module Options_implied_by_dash_p = struct
     ; promote_install_files = true
     }
 
+  let dash_dash_release =
+    let+ (_ : bool) =
+      Arg.(
+        value & flag
+        & info [ "release" ] ~docs ~docv:"PACKAGES"
+            ~doc:
+              "Put $(b,dune) into a reproducible $(i,release) mode. This is in \
+               fact a shorthand for $(b,--root . --ignore-promoted-rules \
+               --no-config --profile release --always-show-command-line \
+               --promote-install-files --default-target @install). You should \
+               use this option for release builds. For instance, you must use \
+               this option in your $(i,<package>.opam) files. Except if you \
+               already use $(b,-p), as $(b,-p) implies this option.")
+    in
+    release_options
+
+  let options =
+    let+ t = one_of options dash_dash_release
+    and+ only_packages =
+      let+ names =
+        Arg.(
+          value
+          & opt (some packages) None
+          & info [ "only-packages" ] ~docs ~docv:"PACKAGES"
+              ~doc:
+                {|Ignore stanzas referring to a package that is not in
+                      $(b,PACKAGES). $(b,PACKAGES) is a comma-separated list
+                      of package names. Note that this has the same effect
+                      as deleting the relevant stanzas from dune files.
+                      It is mostly meant for releases. During development,
+                      it is likely that what you want instead is to
+                      build a particular $(b,<package>.install) target.|})
+      in
+      Option.map names ~f:(fun names ->
+          { Only_packages.names; command_line_option = "only-packages" })
+    in
+    { t with only_packages }
+
+  let dash_p =
+    let+ pkgs, args =
+      Term.with_used_args
+        Arg.(
+          value
+          & opt (some packages) None
+          & info
+              [ "p"; "for-release-of-packages" ]
+              ~docs ~docv:"PACKAGES"
+              ~doc:
+                "Shorthand for $(b,--release --only-packages PACKAGE). You \
+                 must use this option in your $(i,<package>.opam) files, in \
+                 order to build only what's necessary when your project \
+                 contains multiple packages as well as getting reproducible \
+                 builds.")
+    in
+    { release_options with
+      only_packages =
+        Option.map pkgs ~f:(fun names ->
+            { Only_packages.names; command_line_option = List.hd args })
+    }
+
   let term =
     let+ t = one_of options dash_p
     and+ profile =
@@ -304,8 +383,8 @@ module Options_implied_by_dash_p = struct
             ~env:(Arg.env_var ~doc "DUNE_PROFILE")
             ~doc:
               (Printf.sprintf
-                 "Select the build profile, for instance $(b,dev) \
-                  or$(b,release). The default is $(b,%s)."
+                 "Select the build profile, for instance $(b,dev) or \
+                  $(b,release). The default is $(b,%s)."
                  (Profile.to_string Dune.Profile.default)))
     in
     match profile with
@@ -399,6 +478,12 @@ let term =
       value & flag
       & info [ "debug-backtraces" ] ~docs
           ~doc:{|Always print exception backtraces.|})
+  and+ debug_artifact_substitution =
+    Arg.(
+      value & flag
+      & info
+          [ "debug-artifact-substitution" ]
+          ~docs ~doc:"Print debugging info about artifact substitution")
   and+ terminal_persistence =
     Arg.(
       value
@@ -515,7 +600,11 @@ let term =
           ~env:(Arg.env_var ~doc "DUNE_STORE_ORIG_SOURCE_DIR")
           ~doc)
   and+ cache_mode =
-    let doc = "Activate binary cache" in
+    let doc =
+      Printf.sprintf "Activate binary cache (%s). Default is `%s'."
+        (Arg.doc_alts_enum Config.Caching.Mode.all)
+        (Config.Caching.Mode.to_string Config.default.cache_mode)
+    in
     Arg.(
       value
       & opt (some (enum Config.Caching.Mode.all)) None
@@ -548,7 +637,20 @@ let term =
           ~docs
           ~env:(Arg.env_var ~doc "DUNE_CACHE_CHECK_PROBABILITY")
           ~doc)
-  and+ () = build_info in
+  and+ () = build_info
+  and+ instrument_with =
+    let doc =
+      {|"Enable instrumentation by $(b,BACKENDS).
+        $(b,BACKENDS) is a comma-separated list of library names,
+        each one of which must declare an instrumentation backend.|}
+    in
+    Arg.(
+      value
+      & opt (some (list lib_name)) None
+      & info [ "instrument-with" ] ~docs
+          ~env:(Arg.env_var ~doc "DUNE_INSTRUMENT_WITH")
+          ~docv:"BACKENDS" ~doc)
+  in
   let build_dir = Option.value ~default:default_build_dir build_dir in
   let root = Workspace_root.create ~specified_by_user:root in
   let config = config_of_file config_file in
@@ -574,6 +676,7 @@ let term =
   { debug_dep_path
   ; debug_findlib
   ; debug_backtraces
+  ; debug_artifact_substitution
   ; profile
   ; capture_outputs = not no_buffer
   ; workspace_file
@@ -585,10 +688,7 @@ let term =
   ; promote
   ; force
   ; ignore_promoted_rules
-  ; only_packages =
-      Option.map only_packages ~f:(fun s ->
-          Package.Name.Set.of_list
-            (List.map ~f:Package.Name.of_string (String.split s ~on:',')))
+  ; only_packages
   ; x
   ; config
   ; build_dir
@@ -599,6 +699,7 @@ let term =
   ; stats_trace_file
   ; always_show_command_line
   ; promote_install_files
+  ; instrument_with
   }
 
 let term =
