@@ -21,39 +21,39 @@ module Install = Dune.Install
 module Watermarks = Dune.Watermarks
 module Promotion = Dune.Promotion
 module Colors = Dune.Colors
-module Report_error = Dune.Report_error
 module Dune_project = Dune.Dune_project
 module Workspace = Dune.Workspace
 module Cached_digest = Dune.Cached_digest
 module Profile = Dune.Profile
+module Log = Dune_util.Log
 include Common.Let_syntax
 
 let make_cache (config : Config.t) =
   let make_cache () =
-    let handle (Dune_cache.Dedup file) =
+    let command_handler (Cache.Dedup file) =
       match Build_system.get_cache () with
       | None -> Code_error.raise "deduplication message and no caching" []
       | Some caching -> Scheduler.send_dedup caching.cache file
     in
     match config.cache_transport with
     | Config.Caching.Transport.Direct ->
-      Log.info "enable binary cache in direct access mode";
+      Log.info [ Pp.text "enable binary cache in direct access mode" ];
       let cache =
         Result.ok_exn
           (Result.map_error
              ~f:(fun s -> User_error.E (User_error.make [ Pp.text s ]))
-             (Dune_cache.Cache.make ?duplication_mode:config.cache_duplication
-                handle))
+             (Cache.Local.make ?duplication_mode:config.cache_duplication
+                ~command_handler ()))
       in
-      Dune_cache.make_caching (module Dune_cache.Cache) cache
+      Cache.make_caching (module Cache.Local) cache
     | Daemon ->
-      Log.info "enable binary cache in daemon mode";
+      Log.info [ Pp.text "enable binary cache in daemon mode" ];
       let cache =
         Result.ok_exn
-          (Dune_cache_daemon.Client.make
-             ?duplication_mode:config.cache_duplication handle)
+          (Cache.Client.make ?duplication_mode:config.cache_duplication
+             ~command_handler ())
       in
-      Dune_cache.make_caching (module Dune_cache_daemon.Client) cache
+      Cache.make_caching (module Cache.Client) cache
   in
   Fiber.return
     ( match config.cache_mode with
@@ -63,7 +63,7 @@ let make_cache (config : Config.t) =
         ; check_probability = config.cache_check_probability
         }
     | Config.Caching.Mode.Disabled ->
-      Log.info "disable binary cache";
+      Log.info [ Pp.text "disable binary cache" ];
       None )
 
 module Main = struct
@@ -75,18 +75,55 @@ module Main = struct
     in
     let x = Common.x common in
     let profile = Common.profile common in
+    let instrument_with = Common.instrument_with common in
     let capture_outputs = Common.capture_outputs common in
     let ancestor_vcs = (Common.root common).ancestor_vcs in
-    scan_workspace ?workspace_file ?x ?profile ~capture_outputs ~ancestor_vcs ()
+    scan_workspace ?workspace_file ?x ?profile ?instrument_with ~capture_outputs
+      ~ancestor_vcs ()
 
-  let setup ?external_lib_deps_mode common =
+  let setup common =
     let open Fiber.O in
-    let only_packages = Common.only_packages common in
     let* caching = make_cache (Common.config common) in
     let* workspace = scan_workspace common in
+    let only_packages =
+      Option.map (Common.only_packages common)
+        ~f:(fun { Common.Only_packages.names; command_line_option } ->
+          Package.Name.Set.iter names ~f:(fun pkg_name ->
+              if not (Package.Name.Map.mem workspace.conf.packages pkg_name)
+              then
+                let pkg_name = Package.Name.to_string pkg_name in
+                User_error.raise
+                  [ Pp.textf "I don't know about package %s (passed through %s)"
+                      pkg_name command_line_option
+                  ]
+                  ~hints:
+                    (User_message.did_you_mean pkg_name
+                       ~candidates:
+                         ( Package.Name.Map.keys workspace.conf.packages
+                         |> List.map ~f:Package.Name.to_string )));
+          Package.Name.Map.filter workspace.conf.packages ~f:(fun pkg ->
+              let vendored =
+                match Dune.File_tree.find_dir pkg.path with
+                | None -> assert false
+                | Some d -> (
+                  match Dune.File_tree.Dir.status d with
+                  | Vendored -> true
+                  | _ -> false )
+              in
+              let included = Package.Name.Set.mem names pkg.name in
+              if vendored && included then
+                User_error.raise
+                  [ Pp.textf
+                      "Package %s is vendored and so will never be masked. It \
+                       makes no sense to pass it to -p, --only-packages or \
+                       --for-release-of-packages."
+                      (Package.Name.to_string pkg.name)
+                  ];
+              vendored || included))
+    in
     init_build_system workspace
       ~sandboxing_preference:(Common.config common).sandboxing_preference
-      ?caching ?external_lib_deps_mode ?only_packages
+      ?caching ?only_packages
 end
 
 module Scheduler = struct
