@@ -27,7 +27,7 @@ let man =
 
 let info = Term.info "rules" ~doc ~man
 
-let print_rule_makefile ppf (rule : Build_system.Rule.t) =
+let print_rule_makefile ppf (rule : Build_system.Evaluated_rule.t) =
   let action =
     Action.For_shell.Progn
       [ Mkdir (Path.to_string (Path.build rule.dir))
@@ -45,7 +45,7 @@ let print_rule_makefile ppf (rule : Build_system.Rule.t) =
           Format.fprintf ppf "@ %s" (Path.to_string dep)))
     Pp.render_ignore_tags (Action_to_sh.pp action)
 
-let print_rule_sexp ppf (rule : Build_system.Rule.t) =
+let print_rule_sexp ppf (rule : Build_system.Evaluated_rule.t) =
   let sexp_of_action action =
     Action.for_shell action |> Action.For_shell.encode
   in
@@ -107,12 +107,12 @@ let term =
              the given targets.")
   and+ syntax = Syntax.term
   and+ targets = Arg.(value & pos_all string [] & Arg.info [] ~docv:"TARGET") in
-  let out = Option.map ~f:Path.of_string out in
   let targets = List.map ~f:Arg.Dep.file targets in
-  Common.set_common common ~targets;
+  Common.set_common common ~targets ~external_lib_deps_mode:true;
+  let out = Option.map ~f:Path.of_string out in
   Scheduler.go ~common (fun () ->
       let open Fiber.O in
-      let* setup = Import.Main.setup common ~external_lib_deps_mode:true in
+      let* setup = Import.Main.setup common in
       let request =
         match targets with
         | [] ->
