@@ -1,7 +1,7 @@
 open Stdune
 open Import
 
-(** A string that is "2.1.3" but not expanded by [dune subst] *)
+(** A string that is "2.7.0" but not expanded by [dune subst] *)
 let literal_version = "%%" ^ "VERSION%%"
 
 let doc = "Substitute watermarks in source files."
@@ -64,7 +64,7 @@ let term =
   in
   Path.set_root (Path.External.cwd ());
   Path.Build.set_build_dir (Path.Build.Kind.of_string Common.default_build_dir);
-  Console.init config.display;
+  Config.init config;
   Log.init_disabled ();
   Dune.Scheduler.go ~config Watermarks.subst
 
