@@ -5,6 +5,7 @@ type t =
   | Exe
   | Object
   | Shared_object
+  | Plugin
   | Js
 
 let decode =
@@ -14,6 +15,7 @@ let decode =
     ; ("exe", return Exe)
     ; ("object", return Object)
     ; ("shared_object", return Shared_object)
+    ; ("plugin", Dune_lang.Syntax.since Stanza.syntax (2, 4) >>> return Plugin)
     ; ("js", Dune_lang.Syntax.since Stanza.syntax (1, 11) >>> return Js)
     ]
 
@@ -22,6 +24,7 @@ let to_string = function
   | Exe -> "exe"
   | Object -> "object"
   | Shared_object -> "shared_object"
+  | Plugin -> "plugin"
   | Js -> "js"
 
 let to_dyn t =
@@ -30,4 +33,4 @@ let to_dyn t =
 
 let encode t = Dune_lang.unsafe_atom_of_string (to_string t)
 
-let all = [ C; Exe; Object; Shared_object; Js ]
+let all = [ C; Exe; Object; Shared_object; Plugin; Js ]
