@@ -5,6 +5,8 @@ type t =
   | Cmo
   | Cmx
 
+let compare = Poly.compare
+
 let all = [ Cmi; Cmo; Cmx ]
 
 let choose cmi cmo cmx = function
@@ -14,8 +16,6 @@ let choose cmi cmo cmx = function
 
 let ext = choose ".cmi" ".cmo" ".cmx"
 
-let pp fmt p = Format.pp_print_string fmt (ext p)
-
 let source = choose Ml_kind.Intf Impl Impl
 
 module Dict = struct
