@@ -6,7 +6,7 @@ module Kind = struct
   module Opam = struct
     type t =
       { root : string option
-      ; switch : Context_name.t
+      ; switch : string
       }
   end
 
@@ -18,10 +18,7 @@ module Kind = struct
     | Default -> Dyn.Encoder.string "default"
     | Opam o ->
       Dyn.Encoder.(
-        record
-          [ ("root", option string o.root)
-          ; ("switch", Context_name.to_dyn o.switch)
-          ])
+        record [ ("root", option string o.root); ("switch", string o.switch) ])
 end
 
 module Env_nodes = struct
@@ -35,6 +32,36 @@ module Env_nodes = struct
       (Dune_env.Stanza.find env_nodes.workspace ~profile).env_vars
 end
 
+module Program = struct
+  module Name = String
+
+  module Which_path = struct
+    type t = Path.t option
+
+    let equal = Option.equal Path.equal
+
+    let to_dyn = Dyn.Encoder.option Path.to_dyn
+  end
+
+  let programs_for_which_we_prefer_opt_ext =
+    [ "ocamlc"; "ocamldep"; "ocamlmklib"; "ocamlobjinfo"; "ocamlopt" ]
+
+  let best_path dir program =
+    let exe_path program =
+      let fn = Path.relative dir (program ^ Bin.exe) in
+      Option.some_if (Bin.exists fn) fn
+    in
+    if List.mem program ~set:programs_for_which_we_prefer_opt_ext then
+      match exe_path (program ^ ".opt") with
+      | None -> exe_path program
+      | Some _ as path -> path
+    else
+      exe_path program
+
+  let which ~path program =
+    List.find_map path ~f:(fun dir -> best_path dir program)
+end
+
 module T = struct
   type t =
     { name : Context_name.t
@@ -42,7 +69,7 @@ module T = struct
     ; profile : Profile.t
     ; merlin : bool
     ; fdo_target_exe : Path.t option
-    ; disable_dynamically_linked_foreign_archives : bool
+    ; dynamically_linked_foreign_archives : bool
     ; for_host : t option
     ; implicit : bool
     ; build_dir : Path.Build.t
@@ -50,23 +77,22 @@ module T = struct
     ; path : Path.t list
     ; toplevel_path : Path.t option
     ; ocaml_bin : Path.t
-    ; ocaml : Path.t
+    ; ocaml : Action.Prog.t
     ; ocamlc : Path.t
-    ; ocamlopt : Path.t option
-    ; ocamldep : Path.t
-    ; ocamlmklib : Path.t
-    ; ocamlobjinfo : Path.t option
+    ; ocamlopt : Action.Prog.t
+    ; ocamldep : Action.Prog.t
+    ; ocamlmklib : Action.Prog.t
+    ; ocamlobjinfo : Action.Prog.t
     ; env : Env.t
     ; findlib : Findlib.t
     ; findlib_toolchain : Context_name.t option
     ; arch_sixtyfour : bool
-    ; opam_var_cache : (string, string) Table.t
+    ; install_prefix : Path.t Memo.Lazy.Async.t
     ; ocaml_config : Ocaml_config.t
     ; version : Ocaml_version.t
     ; stdlib_dir : Path.t
-    ; ccomp_type : Lib_config.Ccomp_type.t
     ; supports_shared_libraries : Dynlink_supported.By_the_os.t
-    ; which_cache : (string, Path.t option) Table.t
+    ; which : string -> Path.t option
     ; lib_config : Lib_config.t
     }
 
@@ -89,11 +115,11 @@ module T = struct
       ; ("build_dir", Path.Build.to_dyn t.build_dir)
       ; ("toplevel_path", option path t.toplevel_path)
       ; ("ocaml_bin", path t.ocaml_bin)
-      ; ("ocaml", path t.ocaml)
+      ; ("ocaml", Action.Prog.to_dyn t.ocaml)
       ; ("ocamlc", path t.ocamlc)
-      ; ("ocamlopt", option path t.ocamlopt)
-      ; ("ocamldep", path t.ocamldep)
-      ; ("ocamlmklib", path t.ocamlmklib)
+      ; ("ocamlopt", Action.Prog.to_dyn t.ocamlopt)
+      ; ("ocamldep", Action.Prog.to_dyn t.ocamldep)
+      ; ("ocamlmklib", Action.Prog.to_dyn t.ocamlmklib)
       ; ("env", Env.to_dyn (Env.diff t.env Env.initial))
       ; ("findlib_path", list path (Findlib.paths t.findlib))
       ; ("arch_sixtyfour", Bool t.arch_sixtyfour)
@@ -103,9 +129,7 @@ module T = struct
         )
       ; ( "supports_shared_libraries"
         , Bool (Dynlink_supported.By_the_os.get t.supports_shared_libraries) )
-      ; ("opam_vars", Table.to_dyn string t.opam_var_cache)
       ; ("ocaml_config", Ocaml_config.to_dyn t.ocaml_config)
-      ; ("which", Table.to_dyn (option path) t.which_cache)
       ]
 end
 
@@ -113,40 +137,20 @@ include T
 
 let to_dyn_concise t : Dyn.t = Context_name.to_dyn t.name
 
-let compare a b = compare a.name b.name
+let compare a b = Poly.compare a.name b.name
 
-let opam = lazy (Bin.which ~path:(Env.path Env.initial) "opam")
+let opam = Memo.lazy_ (fun () -> Bin.which ~path:(Env.path Env.initial) "opam")
 
-let opam_config_var ~env ~cache var =
-  match Table.find cache var with
-  | Some _ as x -> Fiber.return x
-  | None -> (
-    match Lazy.force opam with
-    | None -> Fiber.return None
-    | Some fn -> (
-      Process.run_capture (Accept Predicate_lang.any) fn ~env
-        [ "config"; "var"; var ]
-      >>| function
-      | Ok s ->
-        let s = String.trim s in
-        Table.set cache var s;
-        Some s
-      | Error _ -> None ) )
-
-let best_prog dir prog =
-  let fn = Path.relative dir (prog ^ ".opt" ^ Bin.exe) in
-  if Bin.exists fn then
-    Some fn
-  else
-    let fn = Path.relative dir (prog ^ Bin.exe) in
-    if Bin.exists fn then
-      Some fn
-    else
-      None
-
-let which ~path prog = List.find_map path ~f:(fun dir -> best_prog dir prog)
-
-let which ~cache ~path x = Table.find_or_add cache x ~f:(which ~path)
+let read_opam_config_var ~env (var : string) : string option Fiber.t =
+  let (_ : Memo.Run.t) = Memo.current_run () in
+  match Memo.Lazy.force opam with
+  | None -> Fiber.return None
+  | Some fn -> (
+    Process.run_capture (Accept Predicate_lang.any) fn ~env
+      [ "config"; "var"; var ]
+    >>| function
+    | Ok s -> Some (String.trim s)
+    | Error _ -> None )
 
 let ocamlpath_sep =
   if Sys.cygwin then
@@ -225,35 +229,75 @@ let check_fdo_support has_native ocfg ~name =
             version_string
         ]
 
+(* We store this so that library such as dune-configurator can read things
+   runtime. Ideally, this should be created on-demand if we run a program linked
+   against configurator, however we currently don't support this kind of
+   "runtime dependencies" so we just do it eagerly. *)
+let write_dot_dune_dir ~build_dir ~ocamlc ~ocaml_config_vars =
+  let dir = Path.build (Path.Build.relative build_dir ".dune") in
+  Path.rm_rf dir;
+  Path.mkdir_p dir;
+  Io.write_file (Path.relative dir Config.dune_keep_fname) "";
+  let ocamlc = Path.to_absolute_filename ocamlc in
+  let ocaml_config_vars = Ocaml_config.Vars.to_list ocaml_config_vars in
+  let () =
+    let open Dune_lang.Encoder in
+    Io.write_lines
+      (Path.relative dir "configurator")
+      (List.map ~f:Dune_lang.to_string
+         (record_fields
+            [ field "ocamlc" string ocamlc
+            ; field_l "ocaml_config_vars" (pair string string) ocaml_config_vars
+            ]))
+  in
+  let () =
+    let csexp =
+      let open Sexp in
+      let ocaml_config_vars =
+        Sexp.List
+          (List.map ocaml_config_vars ~f:(fun (k, v) -> List [ Atom k; Atom v ]))
+      in
+      List
+        [ List [ Atom "ocamlc"; Atom ocamlc ]
+        ; List [ Atom "ocaml_config_vars"; ocaml_config_vars ]
+        ]
+    in
+    let path = Path.relative dir "configurator.v2" in
+    Io.write_file path (Csexp.to_string csexp)
+  in
+  ()
+
 let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
     ~host_context ~host_toolchain ~profile ~fdo_target_exe
-    ~disable_dynamically_linked_foreign_archives =
-  let opam_var_cache = Table.create (module String) 128 in
-  ( match kind with
-  | Opam { root = Some root; _ } -> Table.set opam_var_cache "root" root
-  | _ -> () );
+    ~dynamically_linked_foreign_archives ~instrument_with =
   let prog_not_found_in_path prog =
     Utils.program_not_found prog ~context:name ~loc:None
   in
-  let which_cache = Table.create (module String) 128 in
-  let which x = which ~cache:which_cache ~path x in
+  let which_memo =
+    Memo.create
+      (sprintf "which-memo-for-%s" (Context_name.to_string name))
+      ~input:(module Program.Name)
+      ~output:(Allow_cutoff (module Program.Which_path))
+      ~visibility:Hidden Sync (Program.which ~path)
+  in
+  let which = Memo.exec which_memo in
   let which_exn x =
     match which x with
     | None -> prog_not_found_in_path x
     | Some x -> x
   in
   let findlib_config_path =
-    lazy
-      (let fn = which_exn "ocamlfind" in
-       (* When OCAMLFIND_CONF is set, "ocamlfind printconf" does print the
-          contents of the variable, but "ocamlfind printconf conf" still prints
-          the configuration file set at the configuration time of ocamlfind,
-          sigh... *)
-       ( match Env.get env "OCAMLFIND_CONF" with
-       | Some s -> Fiber.return s
-       | None -> Process.run_capture_line ~env Strict fn [ "printconf"; "conf" ]
-       )
-       >>| Path.of_filename_relative_to_initial_cwd)
+    Memo.lazy_async ~cutoff:Path.equal (fun () ->
+        let fn = which_exn "ocamlfind" in
+        (* When OCAMLFIND_CONF is set, "ocamlfind printconf" does print the
+           contents of the variable, but "ocamlfind printconf conf" still prints
+           the configuration file set at the configuration time of ocamlfind,
+           sigh... *)
+        ( match Env.get env "OCAMLFIND_CONF" with
+        | Some s -> Fiber.return s
+        | None ->
+          Process.run_capture_line ~env Strict fn [ "printconf"; "conf" ] )
+        >>| Path.of_filename_relative_to_initial_cwd)
   in
   let create_one ~(name : Context_name.t) ~implicit ~findlib_toolchain ~host
       ~merlin =
@@ -261,7 +305,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       match findlib_toolchain with
       | None -> Fiber.return None
       | Some toolchain ->
-        let+ path = Lazy.force findlib_config_path in
+        let+ path = Memo.Lazy.Async.force findlib_config_path in
         let toolchain = Context_name.to_string toolchain in
         let context = Context_name.to_string name in
         Some (Findlib.Config.load path ~toolchain ~context)
@@ -271,9 +315,14 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       let* conf = findlib_config in
       let* s = Findlib.Config.get conf prog in
       match Filename.analyze_program_name s with
-      | In_path
+      | In_path -> which s
       | Relative_to_current_dir ->
-        which s
+        User_error.raise
+          [ Pp.textf
+              "The effective Findlib configuration specifies the relative path \
+               %S for the program %S. This is currently not supported."
+              s prog
+          ]
       | Absolute -> Some (Path.of_filename_relative_to_initial_cwd s)
     in
     let ocamlc =
@@ -285,22 +334,21 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
         | None -> prog_not_found_in_path "ocamlc" )
     in
     let dir = Path.parent_exn ocamlc in
-    let ocaml_tool_not_found prog =
-      User_error.raise
-        [ Pp.textf "ocamlc found in %s, but %s/%s doesn't exist (context: %s)"
-            (Path.to_string dir) (Path.to_string dir) prog
-            (Context_name.to_string name)
-        ]
-    in
     let get_ocaml_tool prog =
       match get_tool_using_findlib_config prog with
-      | None -> best_prog dir prog
-      | Some _ as x -> x
-    in
-    let get_ocaml_tool_exn prog =
-      match get_ocaml_tool prog with
-      | None -> ocaml_tool_not_found prog
-      | Some fn -> fn
+      | Some x -> Ok x
+      | None -> (
+        match Program.best_path dir prog with
+        | Some p -> Ok p
+        | None ->
+          let hint =
+            sprintf "ocamlc found in %s, but %s/%s doesn't exist (context: %s)"
+              (Path.to_string dir) (Path.to_string dir) prog
+              (Context_name.to_string name)
+          in
+          Error
+            (Action.Prog.Not_found.create ~context:name ~program:prog ~loc:None
+               ~hint ()) )
     in
     let build_dir = Context_name.build_dir name in
     let ocamlpath =
@@ -322,6 +370,10 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       | None -> []
       | Some s -> Bin.parse_path s ~sep:ocamlpath_sep
     in
+    let opam_config_var_lib =
+      Memo.lazy_async ~cutoff:(Option.equal String.equal) (fun () ->
+          read_opam_config_var ~env "lib")
+    in
     let findlib_paths () =
       match Build_environment_kind.query ~kind ~findlib_toolchain ~env with
       | Cross_compilation_using_findlib_toolchain toolchain ->
@@ -335,7 +387,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
         let p = Path.relative p "lib" in
         Fiber.return (ocamlpath @ [ p ])
       | Opam1_environment -> (
-        opam_config_var ~env ~cache:opam_var_cache "lib" >>| function
+        Memo.Lazy.Async.force opam_config_var_lib >>| function
         | Some s -> ocamlpath @ [ Path.of_filename_relative_to_initial_cwd s ]
         | None -> Utils.program_not_found "opam" ~loc:None )
       | Unknown -> (
@@ -364,7 +416,9 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
           in
           ocaml_config_ok_exn
             ( match Ocaml_config.Vars.of_lines lines with
-            | Ok vars -> Ocaml_config.make vars
+            | Ok vars ->
+              write_dot_dune_dir ~build_dir ~ocamlc ~ocaml_config_vars:vars;
+              Ocaml_config.make vars
             | Error msg -> Error (Ocamlc_config, msg) ))
     in
     let version = Ocaml_version.of_ocaml_config ocfg in
@@ -407,11 +461,13 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
                   (Config.local_install_dir ~context:name)
                   "lib/stublibs"))
         ; extend_var "OCAMLPATH" ~path_sep:ocamlpath_sep local_lib_path
+        ; extend_var "OCAMLTOP_INCLUDE_PATH"
+            (Path.relative local_lib_path "toplevel")
         ; extend_var "OCAMLFIND_IGNORE_DUPS_IN" ~path_sep:ocamlpath_sep
             local_lib_path
         ; extend_var "MANPATH"
             (Path.build (Config.local_install_man_dir ~context:name))
-        ; ("DUNE_CONFIGURATOR", Path.to_string ocamlc)
+        ; ("INSIDE_DUNE", Path.to_absolute_filename (Path.build build_dir))
         ]
       in
       Env.extend env ~vars:(Env.Map.of_list_exn vars)
@@ -431,12 +487,10 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
     in
     let stdlib_dir = Path.of_string (Ocaml_config.standard_library ocfg) in
     let natdynlink_supported = Ocaml_config.natdynlink_supported ocfg in
-    let version = Ocaml_version.of_ocaml_config ocfg in
     let arch_sixtyfour = Ocaml_config.word_size ocfg = 64 in
     let ocamlopt = get_ocaml_tool "ocamlopt" in
-    let ccomp_type = Lib_config.Ccomp_type.of_config ocfg in
     let lib_config =
-      { Lib_config.has_native = Option.is_some ocamlopt
+      { Lib_config.has_native = Result.is_ok ocamlopt
       ; ext_obj = Ocaml_config.ext_obj ocfg
       ; ext_lib = Ocaml_config.ext_lib ocfg
       ; os_type = Ocaml_config.os_type ocfg
@@ -447,11 +501,35 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       ; natdynlink_supported =
           Dynlink_supported.By_the_os.of_bool natdynlink_supported
       ; stdlib_dir
-      ; ccomp_type
+      ; ccomp_type = Ocaml_config.ccomp_type ocfg
+      ; profile
+      ; ocaml_version_string = Ocaml_config.version_string ocfg
+      ; ocaml_version = Ocaml_version.of_ocaml_config ocfg
+      ; instrument_with
       }
     in
     if Option.is_some fdo_target_exe then
       check_fdo_support lib_config.has_native ocfg ~name;
+    let ocaml =
+      let program = "ocaml" in
+      match which program with
+      | Some s -> Ok s
+      | None ->
+        Error (Action.Prog.Not_found.create ~context:name ~program ~loc:None ())
+    in
+    let ocaml_bin = dir in
+    let install_prefix =
+      Memo.lazy_async ~cutoff:Path.equal (fun () ->
+          Fiber.map (read_opam_config_var ~env "prefix") ~f:(function
+            | Some x -> Path.of_filename_relative_to_initial_cwd x
+            | None -> Path.parent_exn ocaml_bin))
+    in
+    let supports_shared_libraries =
+      Ocaml_config.supports_shared_libraries ocfg
+    in
+    let dynamically_linked_foreign_archives =
+      supports_shared_libraries && dynamically_linked_foreign_archives
+    in
     let t =
       { name
       ; implicit
@@ -459,7 +537,7 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       ; profile
       ; merlin
       ; fdo_target_exe
-      ; disable_dynamically_linked_foreign_archives
+      ; dynamically_linked_foreign_archives
       ; env_nodes
       ; for_host = host
       ; build_dir
@@ -468,30 +546,24 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
           Option.map
             (Env.get env "OCAML_TOPLEVEL_PATH")
             ~f:Path.of_filename_relative_to_initial_cwd
-      ; ocaml_bin = dir
-      ; ocaml =
-          ( match which "ocaml" with
-          | Some p -> p
-          | None -> prog_not_found_in_path "ocaml" )
+      ; ocaml_bin
+      ; ocaml
       ; ocamlc
       ; ocamlopt
-      ; ocamldep = get_ocaml_tool_exn "ocamldep"
-      ; ocamlmklib = get_ocaml_tool_exn "ocamlmklib"
-      ; ocamlobjinfo = which "ocamlobjinfo"
+      ; ocamldep = get_ocaml_tool "ocamldep"
+      ; ocamlmklib = get_ocaml_tool "ocamlmklib"
+      ; ocamlobjinfo = get_ocaml_tool "ocamlobjinfo"
       ; env
-      ; findlib =
-          Findlib.create ~stdlib_dir ~paths:findlib_paths ~version ~lib_config
+      ; findlib = Findlib.create ~paths:findlib_paths ~lib_config
       ; findlib_toolchain
       ; arch_sixtyfour
-      ; opam_var_cache
+      ; install_prefix
       ; stdlib_dir
       ; ocaml_config = ocfg
       ; version
-      ; ccomp_type
       ; supports_shared_libraries =
-          Dynlink_supported.By_the_os.of_bool
-            (Ocaml_config.supports_shared_libraries ocfg)
-      ; which_cache
+          Dynlink_supported.By_the_os.of_bool supports_shared_libraries
+      ; which
       ; lib_config
       }
     in
@@ -499,12 +571,12 @@ let create ~(kind : Kind.t) ~path ~env ~env_nodes ~name ~merlin ~targets
       let set prog =
         Response_file.set ~prog (Zero_terminated_strings "-args0")
       in
-      set t.ocaml;
+      Result.iter t.ocaml ~f:set;
       set t.ocamlc;
-      Option.iter t.ocamlopt ~f:set;
-      set t.ocamldep;
+      Result.iter t.ocamlopt ~f:set;
+      Result.iter t.ocamldep ~f:set;
       if Ocaml_version.ocamlmklib_supports_response_file version then
-        set t.ocamlmklib
+        Result.iter ~f:set t.ocamlmklib
     );
     Fiber.return t
   in
@@ -544,14 +616,11 @@ let extend_paths t ~env =
   in
   Env.extend ~vars env
 
-let opam_config_var t var =
-  opam_config_var ~env:t.env ~cache:t.opam_var_cache var
-
 let default ~merlin ~env_nodes ~env ~targets ~fdo_target_exe
-    ~disable_dynamically_linked_foreign_archives =
+    ~dynamically_linked_foreign_archives ~instrument_with =
   let path = Env.path env in
   create ~kind:Default ~path ~env ~env_nodes ~merlin ~targets ~fdo_target_exe
-    ~disable_dynamically_linked_foreign_archives
+    ~dynamically_linked_foreign_archives ~instrument_with
 
 let opam_version =
   let f opam =
@@ -582,9 +651,9 @@ let opam_version =
 
 let create_for_opam ~root ~env ~env_nodes ~targets ~profile ~switch ~name
     ~merlin ~host_context ~host_toolchain ~fdo_target_exe
-    ~disable_dynamically_linked_foreign_archives =
+    ~dynamically_linked_foreign_archives ~instrument_with =
   let opam =
-    match Lazy.force opam with
+    match Memo.Lazy.force opam with
     | None -> Utils.program_not_found "opam" ~loc:None
     | Some fn -> fn
   in
@@ -595,7 +664,7 @@ let create_for_opam ~root ~env ~env_nodes ~targets ~profile ~switch ~name
       ; ( match root with
         | None -> []
         | Some root -> [ "--root"; root ] )
-      ; [ "--switch"; Context_name.to_string switch; "--sexp" ]
+      ; [ "--switch"; switch; "--sexp" ]
       ; ( if version < (2, 0, 0) then
           []
         else
@@ -632,7 +701,8 @@ let create_for_opam ~root ~env ~env_nodes ~targets ~profile ~switch ~name
   create
     ~kind:(Opam { root; switch })
     ~profile ~targets ~path ~env ~env_nodes ~name ~merlin ~host_context
-    ~host_toolchain ~fdo_target_exe ~disable_dynamically_linked_foreign_archives
+    ~host_toolchain ~fdo_target_exe ~dynamically_linked_foreign_archives
+    ~instrument_with
 
 let instantiate_context env (workspace : Workspace.t)
     ~(context : Workspace.Context.t) ~host_context =
@@ -651,7 +721,8 @@ let instantiate_context env (workspace : Workspace.t)
       ; paths
       ; loc = _
       ; fdo_target_exe
-      ; disable_dynamically_linked_foreign_archives
+      ; dynamically_linked_foreign_archives
+      ; instrument_with
       } ->
     let merlin =
       workspace.merlin_context = Some (Workspace.Context.name context)
@@ -666,8 +737,8 @@ let instantiate_context env (workspace : Workspace.t)
     in
     let env = extend_paths ~env paths in
     default ~env ~env_nodes ~profile ~targets ~name ~merlin ~host_context
-      ~host_toolchain ~fdo_target_exe
-      ~disable_dynamically_linked_foreign_archives
+      ~host_toolchain ~fdo_target_exe ~dynamically_linked_foreign_archives
+      ~instrument_with
   | Opam
       { base =
           { targets
@@ -679,7 +750,8 @@ let instantiate_context env (workspace : Workspace.t)
           ; paths
           ; loc = _
           ; fdo_target_exe
-          ; disable_dynamically_linked_foreign_archives
+          ; dynamically_linked_foreign_archives
+          ; instrument_with
           }
       ; switch
       ; root
@@ -688,7 +760,7 @@ let instantiate_context env (workspace : Workspace.t)
     let env = extend_paths ~env paths in
     create_for_opam ~root ~env_nodes ~env ~profile ~switch ~name ~merlin
       ~targets ~host_context ~host_toolchain:toolchain ~fdo_target_exe
-      ~disable_dynamically_linked_foreign_archives
+      ~dynamically_linked_foreign_archives ~instrument_with
 
 module Create = struct
   module Output = struct
@@ -700,35 +772,54 @@ module Create = struct
   let call () : t list Fiber.t =
     let env = Memo.Run.Fdecl.get Global.env in
     let workspace = Workspace.workspace () in
-    let rec contexts : t list Fiber.Once.t Context_name.Map.t Lazy.t =
-      lazy
-        ( List.map workspace.contexts ~f:(fun context ->
-              let contexts =
-                Fiber.Once.create (fun () ->
-                    let* host_context =
-                      match Workspace.Context.host_context context with
-                      | None -> Fiber.return None
-                      | Some context -> (
-                        let+ contexts =
-                          Context_name.Map.find_exn (Lazy.force contexts)
-                            context
-                          |> Fiber.Once.get
-                        in
-                        match contexts with
-                        | [ x ] -> Some x
-                        | [] -> assert false (* checked by workspace *)
-                        | _ :: _ -> assert false
-                        (* target cannot be host *) )
-                    in
-                    instantiate_context env workspace ~context ~host_context)
-              in
-              let name = Workspace.Context.name context in
-              (name, contexts))
-        |> Context_name.Map.of_list_exn )
+
+    (* Contexts can depend on each other, for instance for cross-compilation.
+       When instantiating a context, we need the instantiation of its dependent
+       contexts, and we also need to make sure that each context is instantiated
+       only once.
+
+       To do all that, the following code works in two steps:
+
+       - step 1: build a map from context name to their instantiation.
+       Instantiating a context requires looking up dependent contexts in this
+       map, so the values in the map are ivars to break the cyclic definition
+
+       - step 2: read all instantiation ivars in sequence
+
+       If there was a cycle between contexts, the below code would dead-lock.
+       However, we check at parsing time that there is no such cycle. *)
+    let context_ivars_map =
+      Context_name.Map.of_list_map_exn workspace.contexts ~f:(fun ctx ->
+          (Workspace.Context.name ctx, Fiber.Ivar.create ()))
     in
-    Lazy.force contexts |> Context_name.Map.values
-    |> Fiber.parallel_map ~f:Fiber.Once.get
-    |> Fiber.map ~f:List.concat
+    let* () =
+      Fiber.parallel_iter workspace.contexts ~f:(fun context ->
+          let* contexts =
+            let* host_context =
+              match Workspace.Context.host_context context with
+              | None -> Fiber.return None
+              | Some context -> (
+                let+ contexts =
+                  Fiber.Ivar.read
+                    (Context_name.Map.find_exn context_ivars_map context)
+                in
+                match contexts with
+                | [ x ] -> Some x
+                | [] -> assert false (* checked by workspace *)
+                | _ :: _ -> assert false
+                (* target cannot be host *) )
+            in
+            instantiate_context env workspace ~context ~host_context
+          in
+          let name = Workspace.Context.name context in
+          let ivar = Context_name.Map.find_exn context_ivars_map name in
+          Fiber.Ivar.fill ivar contexts)
+    in
+    let* contexts =
+      Fiber.parallel_map (Context_name.Map.values context_ivars_map)
+        ~f:(fun ivar -> Fiber.Ivar.read ivar)
+    in
+    Fiber.return (List.concat contexts)
 
   let memo =
     Memo.create "create-context" ~doc:"create contexts"
@@ -767,20 +858,13 @@ module DB = struct
         Memo.exec memo name
 end
 
-let which t s = which ~cache:t.which_cache ~path:t.path s
-
-let install_prefix t =
-  opam_config_var t "prefix" >>| function
-  | Some x -> Path.of_filename_relative_to_initial_cwd x
-  | None -> Path.parent_exn t.ocaml_bin
-
 let install_ocaml_libdir t =
   match (t.kind, t.findlib_toolchain, Setup.library_destdir) with
   | Default, None, Some d ->
     Fiber.return (Some (Path.of_filename_relative_to_initial_cwd d))
   | _ -> (
     (* If ocamlfind is present, it has precedence over everything else. *)
-    match which t "ocamlfind" with
+    match t.which "ocamlfind" with
     | Some fn ->
       let+ s =
         Process.run_capture_line ~env:t.env Strict fn [ "printconf"; "destdir" ]
@@ -790,21 +874,31 @@ let install_ocaml_libdir t =
 
 let compiler t (mode : Mode.t) =
   match mode with
-  | Byte -> Some t.ocamlc
+  | Byte -> Ok t.ocamlc
   | Native -> t.ocamlopt
 
 let best_mode t : Mode.t =
   match t.ocamlopt with
-  | Some _ -> Native
-  | None -> Byte
+  | Ok _ -> Native
+  | Error _ -> Byte
 
 let cc_g (ctx : t) =
-  match ctx.ccomp_type with
+  match ctx.lib_config.ccomp_type with
   | Msvc -> []
   | Other _ -> [ "-g" ]
 
 let name t = t.name
 
-let has_native t = Option.is_some t.ocamlopt
+let has_native t = Result.is_ok t.ocamlopt
 
 let lib_config t = t.lib_config
+
+let map_exe (context : t) =
+  match context.for_host with
+  | None -> fun exe -> exe
+  | Some (host : t) -> (
+    fun exe ->
+      match Path.extract_build_context_dir exe with
+      | Some (dir, exe) when Path.equal dir (Path.build context.build_dir) ->
+        Path.append_source (Path.build host.build_dir) exe
+      | _ -> exe )
