@@ -11,15 +11,18 @@ module Name : sig
 
   val compare : t -> t -> Ordering.t
 
+  val equal : t -> t -> bool
+
   val to_dyn : t -> Dyn.t
+
+  val to_string : t -> string
 end
 
 type t
 
 (** A Coq module [a.b.foo] defined in file [a/b/foo.v] *)
 val make :
-     source:
-       Path.Build.t
+     source:Path.Build.t
        (** file = .v source file; module name has to be the same so far *)
   -> prefix:string list (** Library-local qualified prefix *)
   -> name:Name.t (** Name of the module *)
@@ -31,9 +34,11 @@ val source : t -> Path.Build.t
 
 val prefix : t -> string list
 
-val name : t -> string
+val name : t -> Name.t
 
-val obj_file : obj_dir:Path.Build.t -> ext:string -> t -> Path.Build.t
+val dep_file : obj_dir:Path.Build.t -> t -> Path.Build.t
+
+val obj_file : obj_dir:Path.Build.t -> t -> Path.Build.t
 
 val to_dyn : t -> Dyn.t
 
