@@ -6,13 +6,14 @@ type is_component_of_a_group_but_not_the_root =
   }
 
 type t =
-  | Standalone of
-      (File_tree.Dir.t * Stanza.t list Dir_with_dune.t option) option
+  | Generated
+  | Source_only of File_tree.Dir.t
+  | Standalone of File_tree.Dir.t * Stanza.t list Dir_with_dune.t
   (* Directory not part of a multi-directory group. The argument is [None] for
      directory that are not from the source tree, such as generated ones. *)
   | Group_root of
       File_tree.Dir.t
-      * Dune_file.Include_subdirs.qualification
+      * (Loc.t * Dune_file.Include_subdirs.qualification)
       * Stanza.t list Dir_with_dune.t
   (* Directory with [(include_subdirs x)] where [x] is not [no] *)
   | Is_component_of_a_group_but_not_the_root of
