@@ -14,12 +14,25 @@ module Stanza : sig
     val to_string : t -> string
   end
 
+  module Odoc : sig
+    type warnings =
+      | Fatal
+      | Nonfatal
+
+    type t = { warnings : warnings option }
+
+    val decode : t Dune_lang.Decoder.t
+  end
+
   type config =
     { flags : Ocaml_flags.Spec.t
     ; foreign_flags : Ordered_set_lang.Unexpanded.t Foreign.Language.Dict.t
     ; env_vars : Env.t
     ; binaries : File_binding.Unexpanded.t list
     ; inline_tests : Inline_tests.t option
+    ; menhir_flags : Ordered_set_lang.Unexpanded.t
+    ; odoc : Odoc.t
+    ; coq : Ordered_set_lang.Unexpanded.t
     }
 
   type pattern =
