@@ -40,17 +40,27 @@ module Entry : sig
   type t =
     | Library of Lib.t
     | Deprecated_library_name of Deprecated_library_name.t
+    | Hidden_library of Lib.t
+        (** Only for external libraries that:
+
+            - are not built with dune
+
+            - have a [META] file with an unsatisfied [exist_if] field
+
+            Dune itself never produces hidden libraries. *)
 
   val name : t -> Lib_name.t
 
   val version : t -> string option
 
+  val loc : t -> Loc.t
+
   val to_dyn : t Dyn.Encoder.t
 end
 
 type t =
   { name : Package.Name.t
-  ; entries : Entry.t list
+  ; entries : Entry.t Lib_name.Map.t
   ; version : string option
   ; dir : Path.t
   }
