@@ -18,8 +18,7 @@ module Linkage : sig
   (** Native compilation, extension [.exe] *)
   val native : t
 
-  (** Byte compilation, link with [-custom] or [-output-complete-exe], extension
-      [.exe] *)
+  (** Byte compilation with stubs statically linked in, extension [.exe] *)
   val custom : Context.t -> t
 
   (** [native] if supported, [custom] if not *)
@@ -28,7 +27,8 @@ module Linkage : sig
   (** Javascript compilation, extension [.bc.js] *)
   val js : t
 
-  val of_user_config : Context.t -> Dune_file.Executables.Link_mode.t -> t
+  val of_user_config :
+    Context.t -> loc:Loc.t -> Dune_file.Executables.Link_mode.t -> t
 end
 
 (** {1 High-level functions} *)
@@ -41,6 +41,7 @@ val build_and_link :
   -> promote:Rule.Promote.t option
   -> ?link_args:Command.Args.static Command.Args.t Build.t
   -> ?o_files:Path.t list
+  -> ?embed_in_plugin_libraries:(Loc.t * Lib_name.t) list
   -> Compilation_context.t
   -> unit
 
@@ -50,6 +51,7 @@ val build_and_link_many :
   -> promote:Rule.Promote.t option
   -> ?link_args:Command.Args.static Command.Args.t Build.t
   -> ?o_files:Path.t list
+  -> ?embed_in_plugin_libraries:(Loc.t * Lib_name.t) list
   -> Compilation_context.t
   -> unit
 
