@@ -8,34 +8,28 @@ type t
 
 val meta_fn : string
 
-val create :
-     stdlib_dir:Path.t
-  -> paths:Path.t list
-  -> version:Ocaml_version.t
-  -> lib_config:Lib_config.t
-  -> t
+val create : paths:Path.t list -> lib_config:Lib_config.t -> t
 
 (** The search path for this DB *)
 val paths : t -> Path.t list
 
-module Package : sig
-  val preds : Variant.Set.t
-end
+val findlib_predicates_set_by_dune : Variant.Set.t
 
 module Unavailable_reason : sig
   type t =
     | Not_found
         (** The package is hidden because it contains an unsatisfied 'exist_if'
             clause *)
-    | Hidden of Dune_package.Lib.t
     | Invalid_dune_package of exn
 
-  val to_string : t -> string
-
   val to_dyn : t -> Dyn.t
 end
 
-(** Lookup a package in the given database *)
+(** Lookup a whole package, including sub-packages, in the given database.
+    [root_name] must be a library name without dots. *)
+val find_root_package :
+  t -> Package.Name.t -> (Dune_package.t, Unavailable_reason.t) result
+
 val find :
   t -> Lib_name.t -> (Dune_package.Entry.t, Unavailable_reason.t) result
 
@@ -44,12 +38,11 @@ val available : t -> Lib_name.t -> bool
 (** List all the packages available in this Database *)
 val all_packages : t -> Dune_package.Entry.t list
 
-(** List all the packages that are not available in this database *)
-val all_unavailable_packages : t -> (Lib_name.t * Unavailable_reason.t) list
+(** List all the packages that have broken [dune-package] files *)
+val all_broken_packages : t -> (Package.Name.t * exn) list
 
 (** A dummy package. This is used to implement [external-lib-deps] *)
-val dummy_package :
-  t -> name:Lib_name.t -> lib_config:Lib_config.t -> Dune_package.Lib.t
+val dummy_lib : t -> name:Lib_name.t -> Dune_package.Lib.t
 
 module Config : sig
   type t
