@@ -1,3 +1,5 @@
+open Import
+
 (** This module loads and validates foreign sources from directories. *)
 
 type t
@@ -10,10 +12,9 @@ val for_archive : t -> archive_name:Foreign.Archive.Name.t -> Foreign.Sources.t
 
 val for_exes : t -> first_exe:string -> Foreign.Sources.t
 
-(** [make stanzas ~sources ~ext_obj] loads and validates foreign sources. We use
-    [ext_obj] only for nicer error messages. *)
 val make :
      Stanza.t list Dir_with_dune.t
-  -> sources:Foreign.Sources.Unresolved.t
-  -> ext_obj:string
+  -> include_subdirs:Loc.t * Dune_file.Include_subdirs.t
+  -> lib_config:Lib_config.t
+  -> dirs:(Path.Build.t * 'a * String.Set.t) list
   -> t
