@@ -3,8 +3,7 @@
 open! Import
 
 val gen :
-     package:string
-  -> version:string option
-  -> ?add_directory_entry:bool
+     package:Package.t
+  -> add_directory_entry:bool
   -> Super_context.Lib_entry.t list
   -> Meta.t
