@@ -4,7 +4,6 @@ open! Import
 (* Generate rules. Returns evaluated Dune files per context names. *)
 val gen :
      contexts:Context.t list
-  -> ?external_lib_deps_mode:bool (* default: false *)
-  -> ?only_packages:Package.Name.Set.t
+  -> ?only_packages:Package.t Package.Name.Map.t
   -> Dune_load.conf
   -> Super_context.t Context_name.Map.t Fiber.t
