@@ -1,27 +1,21 @@
 open Stdune
 
-module Ccomp_type : sig
-  type t =
-    | Msvc
-    | Other of string
-
-  val to_dyn : t -> Dyn.t
-
-  val of_config : Ocaml_config.t -> t
-end
-
 type t =
   { has_native : bool
   ; ext_lib : string
   ; ext_obj : string
-  ; os_type : string
+  ; os_type : Ocaml_config.Os_type.t
   ; architecture : string
   ; system : string
   ; model : string  (** Native dynlink *)
   ; natdynlink_supported : Dynlink_supported.By_the_os.t
   ; ext_dll : string
   ; stdlib_dir : Path.t
-  ; ccomp_type : Ccomp_type.t
+  ; ccomp_type : Ocaml_config.Ccomp_type.t
+  ; profile : Profile.t
+  ; ocaml_version_string : string
+  ; ocaml_version : Ocaml_version.t
+  ; instrument_with : Lib_name.t list
   }
 
 val allowed_in_enabled_if : (string * Dune_lang.Syntax.Version.t) list
