@@ -1,6 +1,5 @@
 open! Stdune
 open Import
-open Build.O
 open! No_io
 module Buildable = Dune_file.Buildable
 module Library = Dune_file.Library
@@ -20,7 +19,7 @@ let build_lib (lib : Library.t) ~sctx ~dir_contents ~expander ~flags ~dir ~mode
     ~cm_files =
   let ctx = Super_context.context sctx in
   let { Lib_config.ext_lib; _ } = ctx.lib_config in
-  Option.iter (Context.compiler ctx mode) ~f:(fun compiler ->
+  Result.iter (Context.compiler ctx mode) ~f:(fun compiler ->
       let target = Library.archive lib ~dir ~ext:(Mode.compiled_lib_ext mode) in
       let stubs_flags =
         List.concat_map (Library.foreign_archives lib) ~f:(fun archive ->
@@ -35,9 +34,9 @@ let build_lib (lib : Library.t) ~sctx ~dir_contents ~expander ~flags ~dir ~mode
       in
       let map_cclibs =
         (* https://github.com/ocaml/dune/issues/119 *)
-        match ctx.ccomp_type with
+        match ctx.lib_config.ccomp_type with
         | Msvc -> msvc_hack_cclibs
-        | Other _ -> Fn.id
+        | Other _ -> Fun.id
       in
       let obj_deps =
         Build.paths (Cm_files.unsorted_objects_and_cms cm_files ~mode)
@@ -52,7 +51,8 @@ let build_lib (lib : Library.t) ~sctx ~dir_contents ~expander ~flags ~dir ~mode
           ~standard:(Build.return [])
       in
       Super_context.add_rule ~dir sctx ~loc:lib.buildable.loc
-        ( obj_deps
+        (let open Build.With_targets.O in
+        Build.with_no_targets obj_deps
         >>> Command.run (Ok compiler) ~dir:(Path.build ctx.build_dir)
               [ Command.Args.dyn ocaml_flags
               ; A "-a"
@@ -83,7 +83,7 @@ let build_lib (lib : Library.t) ~sctx ~dir_contents ~expander ~flags ~dir ~mode
                       [ Library.archive lib ~dir ~ext:ext_lib ]
                     else
                       [] )
-              ] ))
+              ]))
 
 let gen_wrapped_compat_modules (lib : Library.t) cctx =
   let modules = Compilation_context.modules cctx in
@@ -122,9 +122,6 @@ let ocamlmklib ~loc ~c_library_flags ~sctx ~dir ~expander ~o_files ~archive_name
   let static_target =
     Foreign.Archive.Name.lib_file archive_name ~dir ~ext_lib
   in
-  let dynamic_target =
-    Foreign.Archive.Name.dll_file archive_name ~dir ~ext_dll
-  in
   let build ~custom ~sandbox targets =
     Super_context.add_rule sctx ~sandbox ~dir ~loc
       (let cclibs_args =
@@ -132,7 +129,7 @@ let ocamlmklib ~loc ~c_library_flags ~sctx ~dir ~expander ~o_files ~archive_name
            ~standard:(Build.return [])
        in
        let ctx = Super_context.context sctx in
-       Command.run ~dir:(Path.build ctx.build_dir) (Ok ctx.ocamlmklib)
+       Command.run ~dir:(Path.build ctx.build_dir) ctx.ocamlmklib
          [ A "-g"
          ; ( if custom then
              A "-custom"
@@ -146,7 +143,7 @@ let ocamlmklib ~loc ~c_library_flags ~sctx ~dir ~expander ~o_files ~archive_name
                 case, but we pass them unconditionally for simplicity. *)
              (Build.map cclibs_args ~f:(fun cclibs ->
                   (* https://github.com/ocaml/dune/issues/119 *)
-                  match ctx.ccomp_type with
+                  match ctx.lib_config.ccomp_type with
                   | Msvc ->
                     let cclibs = msvc_hack_cclibs cclibs in
                     Command.quote_args "-ldopt" cclibs
@@ -154,14 +151,17 @@ let ocamlmklib ~loc ~c_library_flags ~sctx ~dir ~expander ~o_files ~archive_name
          ; Hidden_targets targets
          ])
   in
+  let dynamic_target =
+    Foreign.Archive.Name.dll_file archive_name ~dir ~ext_dll
+  in
   if build_targets_together then
     (* Build both the static and dynamic targets in one [ocamlmklib] invocation,
        unless dynamically linked foreign archives are disabled. *)
     build ~sandbox:Sandbox_config.no_special_requirements ~custom:false
-      ( if ctx.disable_dynamically_linked_foreign_archives then
-        [ static_target ]
+      ( if ctx.dynamically_linked_foreign_archives then
+        [ static_target; dynamic_target ]
       else
-        [ static_target; dynamic_target ] )
+        [ static_target ] )
   else (
     (* Build the static target only by passing the [-custom] flag. *)
     build ~sandbox:Sandbox_config.no_special_requirements ~custom:true
@@ -177,7 +177,7 @@ let ocamlmklib ~loc ~c_library_flags ~sctx ~dir ~expander ~o_files ~archive_name
        "optional targets", allowing us to run [ocamlmklib] with the [-failsafe]
        flag, which always produces the static target and sometimes produces the
        dynamic target too. *)
-    if not ctx.disable_dynamically_linked_foreign_archives then
+    if ctx.dynamically_linked_foreign_archives then
       build ~sandbox:Sandbox_config.needs_sandboxing ~custom:false
         [ dynamic_target ]
   )
@@ -190,7 +190,8 @@ let foreign_rules (library : Foreign.Library.t) ~sctx ~expander ~dir
   let archive_name = library.archive_name in
   let o_files =
     let foreign_sources =
-      Dir_contents.foreign_sources_of_archive dir_contents ~archive_name
+      Dir_contents.foreign_sources dir_contents
+      |> Foreign_sources.for_archive ~archive_name
     in
     Foreign_rules.build_o_files ~sctx ~dir ~expander ~requires:(Result.ok [])
       ~dir_contents ~foreign_sources
@@ -207,8 +208,9 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents
   let sctx = Compilation_context.super_context cctx in
   let lib_o_files =
     let foreign_sources =
-      Dir_contents.foreign_sources_of_library dir_contents
-        ~name:(Library.best_name lib)
+      let foreign_sources = Dir_contents.foreign_sources dir_contents in
+      let name = Library.best_name lib in
+      Foreign_sources.for_lib foreign_sources ~name
     in
     Foreign_rules.build_o_files ~sctx ~dir ~expander ~requires ~dir_contents
       ~foreign_sources
@@ -231,7 +233,7 @@ let build_stubs lib ~cctx ~dir ~expander ~requires ~dir_contents
 
 let build_shared lib ~dir_contents ~sctx ~dir ~flags =
   let ctx = Super_context.context sctx in
-  Option.iter ctx.ocamlopt ~f:(fun ocamlopt ->
+  Result.iter ctx.ocamlopt ~f:(fun ocamlopt ->
       let ext_lib = ctx.lib_config.ext_lib in
       let src =
         let ext = Mode.compiled_lib_ext Native in
@@ -247,9 +249,12 @@ let build_shared lib ~dir_contents ~sctx ~dir ~flags =
                let dir = Foreign.Archive.dir_path ~dir archive in
                Command.Args.S [ A "-I"; Path (Path.build dir) ]))
       in
+      let open Build.With_targets.O in
       let build =
-        Build.paths
-          (Library.foreign_lib_files lib ~dir ~ext_lib |> List.map ~f:Path.build)
+        Build.with_no_targets
+          (Build.paths
+             ( Library.foreign_lib_files lib ~dir ~ext_lib
+             |> List.map ~f:Path.build ))
         >>> Command.run ~dir:(Path.build ctx.build_dir) (Ok ocamlopt)
               [ Command.Args.dyn (Ocaml_flags.get flags Native)
               ; A "-shared"
@@ -264,7 +269,8 @@ let build_shared lib ~dir_contents ~sctx ~dir ~flags =
       in
       let build =
         if Lib_archives.has_native_archive lib ctx.lib_config dir_contents then
-          Build.path (Path.build (Library.archive lib ~dir ~ext:ext_lib))
+          Build.with_no_targets
+            (Build.path (Path.build (Library.archive lib ~dir ~ext:ext_lib)))
           >>> build
         else
           build
@@ -294,7 +300,11 @@ let setup_build_archives (lib : Dune_file.Library.t) ~dir_contents ~cctx
                 let src =
                   Path.build (Obj_dir.Module.obj_file obj_dir m ~kind ~ext)
                 in
-                let dst = Path.Build.relative dir (Module.obj_name m ^ ext) in
+                let obj_name = Module.obj_name m in
+                let fname =
+                  Module_name.Unique.artifact_filename obj_name ~ext
+                in
+                let dst = Path.Build.relative dir fname in
                 Super_context.add_rule sctx ~dir (Build.copy ~src ~dst)));
   let top_sorted_modules =
     Dep_graph.top_closed_implementations dep_graphs.impl impl_only
@@ -315,7 +325,7 @@ let setup_build_archives (lib : Dune_file.Library.t) ~dir_contents ~cctx
          Path.Build.relative (Obj_dir.obj_dir obj_dir) (Path.Build.basename src)
          |> Path.Build.extend_basename ~suffix:".js"
        in
-       Js_of_ocaml_rules.build_cm cctx ~js_of_ocaml ~src ~target);
+       Jsoo_rules.build_cm cctx ~js_of_ocaml ~src ~target);
   if Dynlink_supported.By_the_os.get natdynlink_supported && modes.native then
     build_shared ~dir_contents ~sctx lib ~dir ~flags
 
@@ -330,10 +340,15 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope
   let flags = Super_context.ocaml_flags sctx ~dir lib.buildable in
   let obj_dir = Library.obj_dir ~dir lib in
   let vimpl = Virtual_rules.impl sctx ~lib ~scope in
+  let ctx = Super_context.context sctx in
+  let preprocess =
+    Preprocess.Per_module.with_instrumentation lib.buildable.preprocess
+      ~instrumentation_backend:
+        (Lib.DB.instrumentation_backend (Scope.libs scope))
+  in
   (* Preprocess before adding the alias module as it doesn't need preprocessing *)
   let pp =
-    Preprocessing.make sctx ~dir ~dep_kind ~scope
-      ~preprocess:lib.buildable.preprocess ~expander
+    Preprocessing.make sctx ~dir ~dep_kind ~scope ~preprocess ~expander
       ~preprocessor_deps:lib.buildable.preprocessor_deps
       ~lint:lib.buildable.lint
       ~lib_name:(Some (snd lib.name))
@@ -344,8 +359,6 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope
   let modules = Vimpl.impl_modules vimpl modules in
   let requires_compile = Lib.Compile.direct_requires compile_info in
   let requires_link = Lib.Compile.requires_link compile_info in
-  let ctx = Super_context.context sctx in
-  let opaque = Super_context.opaque sctx in
   let dynlink =
     Dynlink_supported.get lib.dynlink ctx.supports_shared_libraries
   in
@@ -354,9 +367,10 @@ let cctx (lib : Library.t) ~sctx ~source_modules ~dir ~expander ~scope
     Dune_file.Mode_conf.Set.eval_detailed lib.modes ~has_native
   in
   Compilation_context.create () ~super_context:sctx ~expander ~scope ~obj_dir
-    ~modules ~flags ~requires_compile ~requires_link ~preprocessing:pp ~opaque
-    ~js_of_ocaml:(Some lib.buildable.js_of_ocaml) ~dynlink ?stdlib:lib.stdlib
-    ~package:(Option.map lib.public ~f:(fun p -> p.package))
+    ~modules ~flags ~requires_compile ~requires_link ~preprocessing:pp
+    ~opaque:Inherit_from_settings ~js_of_ocaml:(Some lib.buildable.js_of_ocaml)
+    ~dynlink ?stdlib:lib.stdlib
+    ~package:(Option.map lib.public ~f:Dune_file.Public_lib.package)
     ?vimpl ~modes
 
 let library_rules (lib : Library.t) ~cctx ~source_modules ~dir_contents
@@ -379,6 +393,11 @@ let library_rules (lib : Library.t) ~cctx ~source_modules ~dir_contents
   gen_wrapped_compat_modules lib cctx;
   Module_compilation.build_all cctx ~dep_graphs;
   let expander = Super_context.expander sctx ~dir in
+  let preprocess =
+    Preprocess.Per_module.with_instrumentation lib.buildable.preprocess
+      ~instrumentation_backend:
+        (Lib.DB.instrumentation_backend (Scope.libs scope))
+  in
   if not (Library.is_virtual lib) then
     setup_build_archives lib ~dir_contents ~cctx ~dep_graphs ~expander;
   let () =
@@ -398,7 +417,7 @@ let library_rules (lib : Library.t) ~cctx ~source_modules ~dir_contents
     };
   ( cctx
   , Merlin.make () ~requires:requires_compile ~flags ~modules
-      ~preprocess:(Buildable.single_preprocess lib.buildable)
+      ~preprocess:(Preprocess.Per_module.single_preprocess preprocess)
       ~libname:(snd lib.name) ~obj_dir )
 
 let rules (lib : Library.t) ~sctx ~dir_contents ~dir ~expander ~scope :
@@ -409,12 +428,15 @@ let rules (lib : Library.t) ~sctx ~dir_contents ~dir ~expander ~scope :
   in
   let f () =
     let source_modules =
-      Dir_contents.modules_of_library dir_contents ~name:(Library.best_name lib)
+      Dir_contents.ocaml dir_contents
+      |> Ml_sources.modules_of_library ~name:(Library.best_name lib)
     in
     let cctx =
       cctx lib ~sctx ~source_modules ~dir ~scope ~expander ~compile_info
     in
     library_rules lib ~cctx ~source_modules ~dir_contents ~compile_info
   in
-  Super_context.Libs.gen_select_rules sctx compile_info ~dir;
-  Super_context.Libs.with_lib_deps sctx compile_info ~dir ~f
+  Buildable_rules.gen_select_rules sctx compile_info ~dir;
+  Buildable_rules.with_lib_deps
+    (Super_context.context sctx)
+    compile_info ~dir ~f
