@@ -22,14 +22,14 @@ val scan_workspace :
   -> ?x:Context_name.t
   -> ?capture_outputs:bool
   -> ?profile:Profile.t
+  -> ?instrument_with:Lib_name.t list
   -> ancestor_vcs:Vcs.t option
   -> unit
   -> workspace Fiber.t
 
 (** Load dune files and initializes the build system *)
 val init_build_system :
-     ?only_packages:Package.Name.Set.t
-  -> ?external_lib_deps_mode:bool
+     ?only_packages:Package.t Package.Name.Map.t
   -> sandboxing_preference:Sandbox_mode.t list
   -> ?caching:Build_system.caching
   -> workspace
