@@ -8,7 +8,7 @@ type t
 val make :
      ?requires:Lib.t list Or_exn.t
   -> flags:Ocaml_flags.t
-  -> ?preprocess:Dune_file.Preprocess.t
+  -> ?preprocess:Preprocess.Without_instrumentation.t Preprocess.t
   -> ?libname:Lib_name.Local.t
   -> ?source_dirs:Path.Source.Set.t
   -> modules:Modules.t
@@ -16,6 +16,8 @@ val make :
   -> unit
   -> t
 
+val merlin_file_name : string
+
 val add_source_dir : t -> Path.Source.t -> t
 
 val merge_all : allow_approx_merlin:bool -> t list -> t option
