@@ -1,3 +1,5 @@
+(** The modules field evaluator interprets the module sources into proper
+    modules in a directory in the context of a stanza from a dune file. *)
 open! Stdune
 
 module Virtual : sig
