@@ -1,7 +1,7 @@
 (** Parse ocamlobjinfo output *)
 open Stdune
 
-type t = Module_name.Set.t Ml_kind.Dict.t
+type t = Module_name.Unique.Set.t Ml_kind.Dict.t
 
 val to_dyn : t -> Dyn.t
 
@@ -9,7 +9,7 @@ val rules :
      dir:Path.Build.t
   -> ctx:Context.t
   -> unit:Path.t
-  -> Action.t Build.t * t Build.t
+  -> Action.t Build.With_targets.t * t Build.t
 
 (** For testing only *)
 val parse : string -> t
