@@ -14,14 +14,14 @@ end = struct
   let of_string sctx s =
     match String.rsplit2 s ~on:'@' with
     | None ->
-      (Lib_name.of_string_exn s ~loc:None, Super_context.public_libs sctx)
+      (Lib_name.parse_string_exn (Loc.none, s), Super_context.public_libs sctx)
     | Some (lib, key) ->
       let scope =
         Dune_project.File_key.of_string key
         |> Super_context.find_project_by_key sctx
         |> Super_context.find_scope_by_project sctx
       in
-      (Lib_name.of_string_exn lib ~loc:None, Scope.libs scope)
+      (Lib_name.parse_string_exn (Loc.none, lib), Scope.libs scope)
 
   let to_string lib project =
     let key = Dune_project.file_key project in
@@ -39,7 +39,7 @@ let lib_unique_name lib =
   | Private project -> Scope_key.to_string name project
 
 let pkg_or_lnu lib =
-  match Lib.package lib with
+  match Lib_info.package (Lib.info lib) with
   | Some p -> Package.Name.to_string p
   | None -> lib_unique_name lib
 
@@ -144,7 +144,13 @@ end
 let odoc sctx =
   SC.resolve_program sctx
     ~dir:(Super_context.build_dir sctx)
-    "odoc" ~loc:None ~hint:"try: opam install odoc"
+    "odoc" ~loc:None ~hint:"opam install odoc"
+
+let odoc_base_flags sctx build_dir =
+  let conf = Super_context.odoc sctx ~dir:build_dir in
+  match conf.Env_node.Odoc.warnings with
+  | Fatal -> Command.Args.A "--warn-error"
+  | Nonfatal -> S []
 
 let module_deps (m : Module.t) ~obj_dir ~(dep_graphs : Dep_graph.Ml_kind.t) =
   Build.dyn_paths_unit
@@ -160,13 +166,15 @@ let module_deps (m : Module.t) ~obj_dir ~(dep_graphs : Dep_graph.Ml_kind.t) =
 let compile_module sctx ~obj_dir (m : Module.t) ~includes:(file_deps, iflags)
     ~dep_graphs ~pkg_or_lnu =
   let odoc_file = Obj_dir.Module.odoc obj_dir m in
+  let open Build.With_targets.O in
   add_rule sctx
-    ( file_deps
-    >>> module_deps m ~obj_dir ~dep_graphs
+    ( Build.with_no_targets file_deps
+    >>> Build.with_no_targets (module_deps m ~obj_dir ~dep_graphs)
     >>>
     let doc_dir = Path.build (Obj_dir.odoc_dir obj_dir) in
     Command.run ~dir:doc_dir (odoc sctx)
       [ A "compile"
+      ; odoc_base_flags sctx odoc_file
       ; A "-I"
       ; Path doc_dir
       ; iflags
@@ -179,14 +187,16 @@ let compile_module sctx ~obj_dir (m : Module.t) ~includes:(file_deps, iflags)
 
 let compile_mld sctx (m : Mld.t) ~includes ~doc_dir ~pkg =
   let odoc_file = Mld.odoc_file m ~doc_dir in
+  let odoc_input = Mld.odoc_input m in
   add_rule sctx
     (Command.run ~dir:(Path.build doc_dir) (odoc sctx)
        [ A "compile"
+       ; odoc_base_flags sctx odoc_input
        ; Command.Args.dyn includes
        ; As [ "--pkg"; Package.Name.to_string pkg ]
        ; A "-o"
        ; Target odoc_file
-       ; Dep (Path.build (Mld.odoc_input m))
+       ; Dep (Path.build odoc_input)
        ]);
   odoc_file
 
@@ -223,15 +233,26 @@ let setup_html sctx (odoc_file : odoc) ~pkg ~requires =
       in
       (odoc_file.html_dir, [ dune_keep ])
   in
+  let open Build.With_targets.O in
   add_rule sctx
-    ( deps
+    ( Build.with_no_targets deps
     >>> Build.progn
-          ( Build.remove_tree to_remove
-          :: Build.mkdir odoc_file.html_dir
+          ( Build.with_no_targets
+              (Build.return
+                 (* Note that we declare no targets apart from [dune_keep]. This
+                    means Dune doesn't know how to build specific documentation
+                    files and that we can't run this rule in a sandbox. To
+                    properly declare targets we would need to support some form
+                    of "dynamic targets" or "target directories". *)
+                 (Action.Progn
+                    [ Action.Remove_tree to_remove
+                    ; Action.Mkdir (Path.build odoc_file.html_dir)
+                    ]))
           :: Command.run
                ~dir:(Path.build (Paths.html_root ctx))
                (odoc sctx)
                [ A "html"
+               ; odoc_base_flags sctx odoc_file.odoc_input
                ; odoc_include_flags ctx pkg requires
                ; A "-o"
                ; Path (Path.build (Paths.html_root ctx))
@@ -254,14 +275,14 @@ let setup_library_odoc_rules cctx (library : Library.t) ~dep_graphs =
   let sctx = Compilation_context.super_context cctx in
   let ctx = Super_context.context sctx in
   let requires = Compilation_context.requires_compile cctx in
+  let info = Lib.info lib in
+  let package = Lib_info.package info in
   let odoc_include_flags =
-    Command.Args.memo (odoc_include_flags ctx (Lib.package lib) requires)
+    Command.Args.memo (odoc_include_flags ctx package requires)
   in
   let obj_dir = Compilation_context.obj_dir cctx in
   let modules = Compilation_context.modules cctx in
-  let includes =
-    (Dep.deps ctx (Lib.package lib) requires, odoc_include_flags)
-  in
+  let includes = (Dep.deps ctx package requires, odoc_include_flags) in
   let modules_and_odoc_files =
     Modules.fold_no_vlib modules ~init:[] ~f:(fun m acc ->
         let compiled =
@@ -270,8 +291,7 @@ let setup_library_odoc_rules cctx (library : Library.t) ~dep_graphs =
         compiled :: acc)
   in
   Dep.setup_deps ctx (Lib local_lib)
-    ( List.map modules_and_odoc_files ~f:(fun (_, p) -> Path.build p)
-    |> Path.Set.of_list )
+    (Path.Set.of_list_map modules_and_odoc_files ~f:(fun (_, p) -> Path.build p))
 
 let setup_css_rule sctx =
   let ctx = Super_context.context sctx in
@@ -412,7 +432,8 @@ let odocs sctx target =
     let dir = Lib_info.src_dir info in
     let modules =
       let name = Lib_info.name info in
-      Dir_contents.get sctx ~dir |> Dir_contents.modules_of_library ~name
+      Dir_contents.get sctx ~dir |> Dir_contents.ocaml
+      |> Ml_sources.modules_of_library ~name
     in
     let obj_dir = Lib_info.obj_dir info in
     Modules.fold_no_vlib modules ~init:[] ~f:(fun m acc ->
@@ -441,7 +462,7 @@ let setup_lib_html_rules_def =
   let f (sctx, lib, requires) =
     let ctx = Super_context.context sctx in
     let odocs = odocs sctx (Lib lib) in
-    let pkg = Lib.package (Lib.Local.to_lib lib) in
+    let pkg = Lib_info.package (Lib.Local.info lib) in
     List.iter odocs ~f:(setup_html sctx ~pkg ~requires);
     let html_files = List.map ~f:(fun o -> Path.build o.html_file) odocs in
     let static_html = List.map ~f:Path.build (static_html ctx) in
@@ -519,15 +540,14 @@ let setup_package_aliases sctx (pkg : Package.t) =
     ( Dep.html_alias ctx (Pkg pkg.name)
       :: ( libs_of_pkg sctx ~pkg:pkg.name
          |> List.map ~f:(fun lib -> Dep.html_alias ctx (Lib lib)) )
-    |> List.map ~f:(fun f -> Path.build (Alias.stamp_file f))
-    |> Path.Set.of_list )
+    |> Path.Set.of_list_map ~f:(fun f -> Path.build (Alias.stamp_file f)) )
 
 let entry_modules_by_lib sctx lib =
   let info = Lib.Local.info lib in
   let dir = Lib_info.src_dir info in
   let name = Lib.name (Lib.Local.to_lib lib) in
-  Dir_contents.get sctx ~dir
-  |> Dir_contents.modules_of_library ~name
+  Dir_contents.get sctx ~dir |> Dir_contents.ocaml
+  |> Ml_sources.modules_of_library ~name
   |> Modules.entry_modules
 
 let entry_modules sctx ~pkg =
@@ -630,9 +650,8 @@ let init sctx =
                  |> Lib.DB.find_even_when_hidden (Scope.libs scope)
                  |> Option.value_exn |> Lib.Local.of_lib_exn |> Option.some )
              | _ -> None))
-    |> List.map ~f:(fun (lib : Lib.Local.t) ->
-           Lib lib |> Dep.html_alias ctx |> Alias.stamp_file |> Path.build)
-    |> Path.Set.of_list )
+    |> Path.Set.of_list_map ~f:(fun (lib : Lib.Local.t) ->
+           Lib lib |> Dep.html_alias ctx |> Alias.stamp_file |> Path.build) )
 
 let gen_rules sctx ~dir:_ rest =
   match rest with
@@ -668,7 +687,7 @@ let gen_rules sctx ~dir:_ rest =
       Lib.Local.of_lib lib
     in
     Option.iter lib ~f:(fun lib ->
-        match Lib.package (Lib.Local.to_lib lib) with
+        match Lib_info.package (Lib.Local.info lib) with
         | None ->
           setup_lib_html_rules sctx lib
             ~requires:(Lib.closure ~linking:false [ Lib.Local.to_lib lib ])
