@@ -75,6 +75,9 @@ end = struct
       { pps; project_root = Option.map project ~f:Dune_project.root }
   end
 
+  (* This mutable table is safe. Even though it can have stale entries remaining
+     from previous runs, the entries themselves are correct, so this seems
+     harmless apart from the lack of error in [decode] in this situation. *)
   let reverse_table : (Digest.t, Decoded.t) Table.t =
     Table.create (module Digest) 128
 
@@ -252,36 +255,31 @@ module Driver = struct
     | Ok _ as x -> x
     | Error No_backend_found ->
       let msg =
-        match libs with
-        | [] -> "You must specify at least one ppx rewriter."
-        | _ -> (
-          match
-            List.filter_map libs ~f:(fun lib ->
-                match Lib_name.to_string (Lib.name lib) with
-                | ("ocaml-migrate-parsetree" | "ppxlib" | "ppx_driver") as s ->
-                  Some s
-                | _ -> None)
-          with
-          | [] ->
-            let pps =
-              match loc with
-              | User_file (_, pps) -> List.map pps ~f:snd
-              | Dot_ppx (_, pps) -> pps
-            in
-            sprintf
-              "No ppx driver were found. It seems that %s %s not compatible \
-               with Dune. Examples of ppx rewriters that are compatible with \
-               Dune are ones using ocaml-migrate-parsetree, ppxlib or \
-               ppx_driver."
-              (String.enumerate_and (List.map pps ~f:Lib_name.to_string))
-              ( match pps with
-              | [ _ ] -> "is"
-              | _ -> "are" )
-          | names ->
-            sprintf
-              "No ppx driver were found.\n\
-               Hint: Try upgrading or reinstalling %s."
-              (String.enumerate_and names) )
+        match
+          List.filter_map libs ~f:(fun lib ->
+              match Lib_name.to_string (Lib.name lib) with
+              | ("ocaml-migrate-parsetree" | "ppxlib" | "ppx_driver") as s ->
+                Some s
+              | _ -> None)
+        with
+        | [] ->
+          let pps =
+            match loc with
+            | User_file (_, pps) -> List.map pps ~f:snd
+            | Dot_ppx (_, pps) -> pps
+          in
+          sprintf
+            "No ppx driver were found. It seems that %s %s not compatible with \
+             Dune. Examples of ppx rewriters that are compatible with Dune are \
+             ones using ocaml-migrate-parsetree, ppxlib or ppx_driver."
+            (String.enumerate_and (List.map pps ~f:Lib_name.to_string))
+            ( match pps with
+            | [ _ ] -> "is"
+            | _ -> "are" )
+        | names ->
+          sprintf
+            "No ppx driver were found.\nHint: Try upgrading or reinstalling %s."
+            (String.enumerate_and names)
       in
       make_error loc msg
     | Error (Too_many_backends ts) ->
@@ -303,7 +301,7 @@ let build_ppx_driver sctx ~dep_kind ~target ~pps ~pp_names =
     | Byte -> Byte_with_stubs_statically_linked_in
     | Native -> Native
   in
-  let compiler = Option.value_exn (Context.compiler ctx mode) in
+  let compiler = Context.compiler ctx mode in
   let jbuild_driver, pps, pp_names = (None, pps, pp_names) in
   let driver_and_libs =
     let open Result.O in
@@ -324,15 +322,18 @@ let build_ppx_driver sctx ~dep_kind ~target ~pps ~pp_names =
   let dir = Path.Build.parent_exn target in
   let ml = Path.Build.relative dir "_ppx.ml" in
   let add_rule ~sandbox = SC.add_rule ~sandbox sctx ~dir in
+  let open Build.With_targets.O in
   add_rule ~sandbox:Sandbox_config.default
     ( Build.of_result_map driver_and_libs ~f:(fun (driver, _) ->
           Build.return (sprintf "let () = %s ()\n" driver.info.main))
     |> Build.write_file_dyn ml );
   add_rule ~sandbox:Sandbox_config.no_special_requirements
-    ( Build.record_lib_deps
-        (Lib_deps.info ~kind:dep_kind (Lib_deps.of_pps pp_names))
-    >>> Command.run (Ok compiler) ~dir:(Path.build ctx.build_dir)
-          [ A "-o"
+    ( Build.with_no_targets
+        (Build.record_lib_deps
+           (Lib_deps.info ~kind:dep_kind (Lib_deps.of_pps pp_names)))
+    >>> Command.run compiler ~dir:(Path.build ctx.build_dir)
+          [ A "-g"
+          ; A "-o"
           ; Target target
           ; A "-w"
           ; A "-24"
@@ -347,7 +348,7 @@ let build_ppx_driver sctx ~dep_kind ~target ~pps ~pp_names =
               (Result.map driver_and_libs ~f:(fun (_driver, libs) ->
                    Command.Args.S
                      [ Lib.L.compile_and_link_flags ~mode:link_mode
-                         ~compile:libs ~link:libs ~lib_config:ctx.lib_config
+                         ~compile:libs ~link:libs
                      ; Hidden_deps
                          (Lib_file_deps.deps libs ~groups:[ Cmi; Cmx ])
                      ]))
@@ -464,31 +465,32 @@ let workspace_root_var = String_with_vars.virt_var __POS__ "workspace_root"
 let promote_correction fn build ~suffix =
   Build.progn
     [ build
-    ; Build.return
-        (Action.diff ~optional:true fn (Path.extend_basename fn ~suffix))
+    ; Build.with_no_targets
+        (Build.return
+           (Action.diff ~optional:true (Path.build fn)
+              (Path.Build.extend_basename fn ~suffix)))
     ]
 
 let chdir action = Action_unexpanded.Chdir (workspace_root_var, action)
 
-let action_for_pp sctx ~dep_kind ~loc ~expander ~action ~src ~target =
+let action_for_pp ~dep_kind ~loc ~expander ~action ~src ~target =
   let action = chdir action in
   let bindings = Pform.Map.input_file (Path.build src) in
   let expander = Expander.add_bindings expander ~bindings in
-  let targets = Expander.Targets.Forbidden "preprocessing actions" in
+  let targets = Targets.Or_forbidden.Forbidden "preprocessing actions" in
   let targets_dir = Option.value ~default:src target |> Path.Build.parent_exn in
   let action =
-    SC.Action.run sctx action ~loc ~expander ~dep_kind ~targets ~targets_dir
+    Action_unexpanded.expand action ~loc ~expander ~dep_kind ~targets
+      ~targets_dir
       (let+ () = Build.path (Path.build src) in
        Bindings.empty)
   in
-  let+ action =
-    match target with
-    | None -> action
-    | Some dst -> Build.action_dyn ~targets:[ dst ] action
-  in
   match target with
   | None -> action
-  | Some dst -> Action.with_stdout_to dst action
+  | Some dst ->
+    Build.With_targets.map
+      ~f:(Action.with_stdout_to dst)
+      (Build.With_targets.add ~targets:[ dst ] action)
 
 (* Generate rules for the dialect modules in [modules] and return a a new module
    with only OCaml sources *)
@@ -503,10 +505,25 @@ let setup_dialect_rules sctx ~dir ~dep_kind ~expander (m : Module.t) =
           Option.value_exn (Module.file ml ~ml_kind) |> Path.as_in_build_dir_exn
         in
         SC.add_rule sctx ~dir
-          (action_for_pp sctx ~dep_kind ~loc ~expander ~action ~src
+          (action_for_pp ~dep_kind ~loc ~expander ~action ~src
              ~target:(Some dst)));
   ml
 
+let add_corrected_suffix_binding expander suffix =
+  let bindings =
+    Pform.Map.singleton "corrected-suffix" (Values [ String suffix ])
+  in
+  Expander.add_bindings expander ~bindings
+
+let driver_flags expander ~flags ~corrected_suffix ~driver_flags ~standard =
+  let args : _ Command.Args.t = S [ As flags ] in
+  let ppx_flags =
+    let expander = add_corrected_suffix_binding expander corrected_suffix in
+    Build.memoize "ppx flags"
+      (Expander.expand_and_eval_set expander driver_flags ~standard)
+  in
+  (ppx_flags, args)
+
 let lint_module sctx ~dir ~expander ~dep_kind ~lint ~lib_name ~scope =
   Staged.stage
     (let alias = Alias.lint ~dir in
@@ -524,7 +541,7 @@ let lint_module sctx ~dir ~expander ~dep_kind ~lint ~lib_name ~scope =
              Module.iter source ~f:(fun _ (src : Module.File.t) ->
                  let src = Path.as_in_build_dir_exn src.path in
                  add_alias src ~loc:(Some loc)
-                   (action_for_pp sctx ~dep_kind ~loc ~expander ~action ~src
+                   (action_for_pp ~dep_kind ~loc ~expander ~action ~src
                       ~target:None))
          | Pps { loc; pps; flags; staged } ->
            if staged then
@@ -533,30 +550,25 @@ let lint_module sctx ~dir ~expander ~dep_kind ~lint ~lib_name ~scope =
            let corrected_suffix = ".lint-corrected" in
            let driver_and_flags =
              let open Result.O in
-             let+ exe, driver, driver_flags =
+             let+ exe, driver, flags =
                ppx_driver_and_flags sctx ~expander ~loc ~lib_name ~flags ~scope
                  pps
              in
-             let flags =
-               let bindings =
-                 Pform.Map.singleton "corrected-suffix"
-                   (Values [ String corrected_suffix ])
-               in
-               let expander = Expander.add_bindings expander ~bindings in
-               Build.memoize "ppx flags"
-                 (Expander.expand_and_eval_set expander driver.info.lint_flags
-                    ~standard:(Build.return []))
+             let ppx_flags, args =
+               driver_flags expander ~flags ~corrected_suffix
+                 ~driver_flags:driver.info.lint_flags
+                 ~standard:(Build.return [])
              in
-             let args : _ Command.Args.t = S [ As driver_flags ] in
-             (exe, flags, args)
+             (exe, ppx_flags, args)
            in
            fun ~source ~ast ->
              Module.iter ast ~f:(fun ml_kind src ->
                  add_alias src.path ~loc:None
                    (promote_correction ~suffix:corrected_suffix
-                      (Option.value_exn (Module.file source ~ml_kind))
-                      (Build.of_result_map driver_and_flags
-                         ~f:(fun (exe, flags, args) ->
+                      (Path.as_in_build_dir_exn
+                         (Option.value_exn (Module.file source ~ml_kind)))
+                      (Build.With_targets.of_result_map ~targets:[]
+                         driver_and_flags ~f:(fun (exe, flags, args) ->
                            Command.run
                              ~dir:(Path.build (SC.build_dir sctx))
                              (Ok (Path.build exe))
@@ -577,11 +589,11 @@ let make sctx ~dir ~expander ~dep_kind ~lint ~preprocess ~preprocessor_deps
     ~lib_name ~scope =
   let preprocess =
     Module_name.Per_item.map preprocess ~f:(fun pp ->
-        Dune_file.Preprocess.remove_future_syntax ~for_:Compiler pp
+        Preprocess.remove_future_syntax ~for_:Compiler pp
           (Super_context.context sctx).version)
   in
   let preprocessor_deps =
-    SC.Deps.interpret sctx preprocessor_deps ~expander
+    Dep_conf_eval.unnamed preprocessor_deps ~expander
     |> Build.memoize "preprocessor deps"
   in
   let lint_module =
@@ -600,10 +612,12 @@ let make sctx ~dir ~expander ~dep_kind ~lint ~preprocess ~preprocessor_deps
           let ast =
             pped_module m ~f:(fun _kind src dst ->
                 let action =
-                  action_for_pp sctx ~dep_kind ~loc ~expander ~action ~src
+                  action_for_pp ~dep_kind ~loc ~expander ~action ~src
                     ~target:(Some dst)
                 in
-                SC.add_rule sctx ~loc ~dir (preprocessor_deps >>> action))
+                let open Build.With_targets.O in
+                SC.add_rule sctx ~loc ~dir
+                  (Build.with_no_targets preprocessor_deps >>> action))
             |> setup_dialect_rules sctx ~dir ~dep_kind ~expander
           in
           if lint then lint_module ~ast ~source:m;
@@ -617,29 +631,26 @@ let make sctx ~dir ~expander ~dep_kind ~lint ~preprocess ~preprocessor_deps
               ppx_driver_and_flags sctx ~expander ~loc ~lib_name ~flags ~scope
                 pps
             in
-            let args : _ Command.Args.t = S [ As flags ] in
-            ( exe
-            , (let bindings =
-                 Pform.Map.singleton "corrected-suffix"
-                   (Values [ String corrected_suffix ])
-               in
-               let expander = Expander.add_bindings expander ~bindings in
-               Build.memoize "ppx flags"
-                 (Expander.expand_and_eval_set expander driver.info.flags
-                    ~standard:(Build.return [ "--as-ppx" ])))
-            , args )
+            let ppx_flags, args =
+              driver_flags expander ~flags ~corrected_suffix
+                ~driver_flags:driver.info.flags
+                ~standard:(Build.return [ "--as-ppx" ])
+            in
+            (exe, ppx_flags, args)
           in
           fun m ~lint ->
+            let open Build.With_targets.O in
             let ast = setup_dialect_rules sctx ~dir ~dep_kind ~expander m in
             if lint then lint_module ~ast ~source:m;
             pped_module ast ~f:(fun ml_kind src dst ->
                 SC.add_rule ~sandbox:Sandbox_config.no_special_requirements sctx
                   ~loc ~dir
                   (promote_correction ~suffix:corrected_suffix
-                     (Option.value_exn (Module.file m ~ml_kind))
-                     ( preprocessor_deps
-                     >>> Build.of_result_map driver_and_flags ~targets:[ dst ]
-                           ~f:(fun (exe, flags, args) ->
+                     (Path.as_in_build_dir_exn
+                        (Option.value_exn (Module.file m ~ml_kind)))
+                     ( Build.with_no_targets preprocessor_deps
+                     >>> Build.With_targets.of_result_map driver_and_flags
+                           ~targets:[ dst ] ~f:(fun (exe, flags, args) ->
                              Command.run
                                ~dir:(Path.build (SC.build_dir sctx))
                                (Ok (Path.build exe))
