@@ -5,6 +5,33 @@ type t =
   | Release
   | User_defined of string
 
+include (
+  Stringlike.Make (struct
+    type nonrec t = t
+
+    let description = "profile"
+
+    let module_ = "Profile"
+
+    let of_string_opt p =
+      (* TODO actually validate *)
+      Some
+        ( match p with
+        | "dev" -> Dev
+        | "release" -> Release
+        | s -> User_defined s )
+
+    let description_of_valid_string = None
+
+    let hint_valid = None
+
+    let to_string = function
+      | Dev -> "dev"
+      | Release -> "release"
+      | User_defined s -> s
+  end) :
+    Stringlike_intf.S with type t := t )
+
 let equal x y =
   match (x, y) with
   | Dev, Dev -> true
@@ -12,16 +39,6 @@ let equal x y =
   | User_defined x, User_defined y -> String.equal x y
   | _, _ -> false
 
-let of_string = function
-  | "dev" -> Dev
-  | "release" -> Release
-  | s -> User_defined s
-
-let to_string = function
-  | Dev -> "dev"
-  | Release -> "release"
-  | User_defined s -> s
-
 let default = Dev
 
 let is_dev = function
@@ -36,11 +53,6 @@ let is_inline_test = function
   | Release -> false
   | _ -> true
 
-let decode =
-  let open Dune_lang.Decoder in
-  let+ name = string in
-  of_string name
-
 let to_dyn =
   let open Dyn.Encoder in
   function
