@@ -1,6 +1,5 @@
 (** Defines build profile for dune. Only one profile is active per context. Some
     profiles are treat specially by dune. *)
-open Stdune
 
 type t =
   | Dev
@@ -15,12 +14,6 @@ val is_release : t -> bool
 
 val is_inline_test : t -> bool
 
-val to_string : t -> string
-
-val of_string : string -> t (* TODO add error handling *)
+include Stringlike_intf.S with type t := t
 
 val default : t
-
-val decode : t Dune_lang.Decoder.t
-
-val to_dyn : t -> Dyn.t
