@@ -10,8 +10,6 @@ module Info : sig
     | Source_file_copy
 
   val of_loc_opt : Loc.t option -> t
-
-  val loc : t -> Loc.t option
 end
 
 module Promote : sig
@@ -46,17 +44,35 @@ module Mode : sig
     | Ignore_source_files
 end
 
-type t =
-  { context : Context.t option
+module Id : sig
+  type t
+
+  val compare : t -> t -> Ordering.t
+
+  module Map : Map.S with type key = t
+
+  module Set : Set.S with type elt = t
+end
+
+type t = private
+  { id : Id.t
+  ; context : Context.t option
   ; env : Env.t option
-  ; build : Action.t Build.t
-  ; targets : Path.Build.Set.t
+  ; action : Action.t Build.With_targets.t
   ; mode : Mode.t
   ; locks : Path.t list
-  ; info : Info.t  (** Directory where all the targets are produced *)
-  ; dir : Path.Build.t
+  ; info : Info.t
+  ; (* Directory where all the targets are produced. *) dir : Path.Build.t
   }
 
+module Set : Set.S with type elt = t
+
+val equal : t -> t -> bool
+
+val hash : t -> int
+
+val to_dyn : t -> Dyn.t
+
 val make :
      ?sandbox:Sandbox_config.t
   -> ?mode:Mode.t
@@ -64,5 +80,16 @@ val make :
   -> env:Env.t option
   -> ?locks:Path.t list
   -> ?info:Info.t
-  -> Action.t Build.t
+  -> Action.t Build.With_targets.t
   -> t
+
+val with_prefix : t -> build:unit Build.t -> t
+
+val loc : t -> Loc.t
+
+val effective_env : t -> Env.t
+
+(** [find_source_dir rule] is the closest source directory corresponding to
+    rule.dir. Eg. [src/dune] for a rule with dir
+    [_build/default/src/dune/.dune.objs]. *)
+val find_source_dir : t -> File_tree.Dir.t
