@@ -23,6 +23,8 @@ module Status : sig
       ; normal : 'a
       }
 
+    val merge : 'a t -> 'b t -> f:('a -> 'b -> 'c) -> 'c t
+
     val find : 'a t -> status -> 'a
 
     val to_dyn : ('a -> Dyn.t) -> 'a t -> Dyn.t
@@ -38,6 +40,10 @@ module Status : sig
   end
 end
 
+type subdir_stanzas = (Loc.t * Predicate_lang.Glob.t) option Status.Map.t
+
+val or_default : subdir_stanzas -> Predicate_lang.Glob.t Status.Map.t
+
 val default : Predicate_lang.Glob.t Status.Map.t
 
 type status_map
@@ -46,6 +52,21 @@ val eval : Predicate_lang.Glob.t Status.Map.t -> dirs:string list -> status_map
 
 val status : status_map -> dir:string -> Status.Or_ignored.t
 
-val decode :
-  (Predicate_lang.Glob.t Status.Map.t * Dune_lang.Ast.t list)
-  Dune_lang.Decoder.t
+module Dir_map : sig
+  type t
+
+  type per_dir =
+    { sexps : Dune_lang.Ast.t list
+    ; subdir_status : subdir_stanzas
+    }
+
+  val descend : t -> string -> t option
+
+  val sub_dirs : t -> string list
+
+  val merge : t -> t -> t
+
+  val root : t -> per_dir
+end
+
+val decode : file:Path.Source.t -> Dir_map.t Dune_lang.Decoder.t
