@@ -28,4 +28,4 @@ val vlib_stubs_o_files : t option -> Path.t list
 
 val impl_cm_kind : t -> Cm_kind.t
 
-val vlib_obj_map : t -> Module.Name_map.t
+val vlib_obj_map : t -> Module.t Module_name.Unique.Map.t
